@@ -52,7 +52,7 @@ int find_teleport(struct Coords *coords, int teleportnumber, int teleportnumber2
 int is_robbo_killed(void);
 void kill_robbo(void);
 void redraw_field(int x, int y);
-void clear_field(struct object *obj);
+void clear_field(int x, int y);
 
 
 /***************************************************************************
@@ -77,7 +77,7 @@ void update_game(void) {
 	struct Coords coords, coords_temp, dest, coords_side_behind, coords_behind;
 	int x, y, i, forceforward;
 	
-	dest.x = 0; // to avoid warnings
+	dest.x = 0; /* to avoid warnings */
 	dest.y = 0;
 	/* Firstly, update Robbo */
 	if (is_robbo_killed()) kill_robbo();	/* If Robbo has been killed then mark everything to be blown-up */
@@ -232,7 +232,7 @@ void update_game(void) {
 									temp_state = board[x_tmp][y].state;
 									temp_blowed = board[x_tmp][y].blowed;
 									temp_direction = board[x_tmp][y].direction;
-									clear_field(&board[x_tmp][y]);
+									clear_field(x_tmp,y);
 									negate_state(x_tmp,y);
 									flag = 1;
 								}
@@ -282,7 +282,7 @@ void update_game(void) {
 									temp_state = board[x_tmp][y].state;
 									temp_blowed = board[x_tmp][y].blowed;
 									temp_direction = board[x_tmp][y].direction;
-									clear_field(&board[x_tmp][y]);
+									clear_field(x_tmp,y);
 									negate_state(x_tmp,y);
 									flag = 1;
 								}
@@ -394,7 +394,7 @@ void update_game(void) {
 									case CAPSULE:
 										break;
 									default:
-										clear_field(&board[coords.x][coords.y]);
+										clear_field(coords.x,coords.y);
 										create_object(coords.x,coords.y,BLASTER);
 										board[coords.x][coords.y].direction = board[x][y].direction;
 										board[coords.x][coords.y].direction2 = board[x][y].direction2;
@@ -407,7 +407,7 @@ void update_game(void) {
 								board[x][y].moved = DELAY_BLASTER;
 							}
 							else {
-								clear_field(&board[x][y]);
+								clear_field(x,y);
 							}
 							break;
 						/*********************/
@@ -429,7 +429,7 @@ void update_game(void) {
 								board[x][y].moved = DELAY_LITTLE_BOOM;
 							}
 							else
-								clear_field(&board[x][y]);
+								clear_field(x,y);
 							break;
 						/*********************/
 						/* LASER_L/D logic   */
@@ -456,12 +456,12 @@ void update_game(void) {
 								}
 								else {														/* blow object if it's destroyable */
 									if(!update_coords(&coords,board[x][y].direction) && board[coords.x][coords.y].destroyable) {
-										clear_field(&board[x][y]);
+										clear_field(x,y);
 										board[coords.x][coords.y].blowed = 1;
 										redraw_field(coords.x, coords.y);
 									}
 									else {
-										clear_field(&board[x][y]);							/* clear lasertrack */
+										clear_field(x,y);							/* clear lasertrack */
 										create_object(x,y,LITTLE_BOOM);
 										board[x][y].moved = DELAY_LITTLE_BOOM;
 										break;
@@ -478,7 +478,7 @@ void update_game(void) {
 									if(board[x][y].returnlaser == 1) {
 										set_coords(&coords,x,y);					/* checking if laser is near gun */
 										update_coords(&coords,(board[x][y].direction+2) & 0x03);
-										clear_field(&board[x][y]);
+										clear_field(x,y);
 									
 										if(board[coords.x][coords.y].type == GUN) {
 											create_object(x,y,LITTLE_BOOM);
@@ -518,7 +518,7 @@ void update_game(void) {
 							} else {
 								i = board[x][y].id_questionmark;	/* This is why create_object() doesn't clear this property */
 								if((i > 0) && robbo.alive) {
-									clear_field(&board[x][y]);
+									clear_field(x,y);
 									if (i == QUESTIONMARK) board[x][y].id_questionmark = random_id();
 									create_object(x,y,i);
 									switch(i) {
@@ -534,7 +534,7 @@ void update_game(void) {
 											break;
 									}
 								} else
-									clear_field(&board[x][y]);
+									clear_field(x,y);
 							}
 							break;
 						/*********************/
@@ -553,7 +553,7 @@ void update_game(void) {
 								board[x][y].state++;
 								board[x][y].moved = DELAY_TELEPORTING;
 							} else {
-								clear_field(&board[x][y]);
+								clear_field(x,y);
 							}
 							break;
 						/*********************/
@@ -655,7 +655,7 @@ void update_game(void) {
 												board[coords.x][coords.y].shooted = DELAY_LASER;
 												break;
 											default:
-												clear_field(&board[coords_temp.x][coords_temp.y]);
+												clear_field(coords_temp.x,coords_temp.y);
 												create_object(coords_temp.x, coords_temp.y,BLASTER);
 												board[coords_temp.x][coords_temp.y].direction = board[x][y].direction;
 												board[coords_temp.x][coords_temp.y].direction2 = board[x][y].direction2;
@@ -1090,7 +1090,7 @@ void move_object(int x,int y,struct Coords coords)
 	board[x1][y1].redraw = TRUE;
 
 	set_images(board[x1][y1].type, x1, y1);
-	clear_field(&board[x][y]);
+	clear_field(x,y);
 	
 }
 
@@ -1132,12 +1132,12 @@ void shoot_object(int x, int y, int direction)
 	switch(board[coords.x][coords.y].type) {
 		case EMPTY_FIELD:			
 			if(direction == 0 || direction == 2) {
-				clear_field(&board[coords.x][coords.y]);
+				clear_field(coords.x,coords.y);
 				create_object(coords.x,coords.y,LASER_L);
 			}
 			else
 			{
-				clear_field(&board[coords.x][coords.y]);
+				clear_field(coords.x,coords.y);
 				create_object(coords.x,coords.y,LASER_D);
 			}
 			
@@ -1217,7 +1217,6 @@ void blow_bomb(int x,int y)
 		/* Blow-up the object */
 		if (!coords_out_of_range(coords) && !board[coords.x][coords.y].blowed && board[coords.x][coords.y].blowable) {
 			if (board[coords.x][coords.y].type == GUN) {
-				/*printf("%i: Blowing-up a gun at %i.%i solidlaser=%i\n", cycle_count, coords.x, coords.y, board[coords.x][coords.y].solidlaser);	 temp temp */
 				board[coords.x][coords.y].blowed = 1;
 				board[coords.x][coords.y].moved = DELAY_BOMB_TARGET;
 				/* Thunor: In Atari Robbo the solid laser fire is left live when the gun is destroyed which is a
@@ -1227,7 +1226,6 @@ void blow_bomb(int x,int y)
 					while (!update_coords(&laserpath, (board[coords.x][coords.y].direction)) &&
 						(board[laserpath.x][laserpath.y].type == LASER_D || board[laserpath.x][laserpath.y].type == LASER_L) &&
 						board[laserpath.x][laserpath.y].solidlaser && board[laserpath.x][laserpath.y].direction == board[coords.x][coords.y].direction) {
-						/*printf("%i: Blowing-up solid laser fire at %i.%i\n", cycle_count, laserpath.x, laserpath.y);	 temp temp */
 						board[laserpath.x][laserpath.y].blowed = 1;
 						board[laserpath.x][laserpath.y].moved = DELAY_BOMB_TARGET;
 					}
@@ -1235,7 +1233,6 @@ void blow_bomb(int x,int y)
 			/* Thunor: In Atari Robbo the solid laser is not blowable but in GNU Robbo there
 			   is no solid laser type, so I need to check the solidlaser property here */
 			} else if (!((board[coords.x][coords.y].type == LASER_D || board[coords.x][coords.y].type == LASER_L) && board[coords.x][coords.y].solidlaser)) {
-				/*printf("%i: Blowing-up a %i at %i.%i\n", cycle_count, board[coords.x][coords.y].type, coords.x, coords.y);	temp temp */
 				board[coords.x][coords.y].blowed = 1;
 				board[coords.x][coords.y].moved = DELAY_BOMB_TARGET;
 				board[coords.x][coords.y].id_questionmark = 0;	/* blowed questionmark doesn't uncover */
@@ -1360,23 +1357,23 @@ void move_robbo(int x,int y)
 			if(robbo.screws == 0)
 				open_exit();
 			scoreline.redraw |= SCORELINE_SCREWS ;
-			clear_field(&board[x_tmp][y_tmp]);
+			clear_field(x_tmp,y_tmp);
 			break;
 		case BULLET:
 			robbo.bullets += 9;
 			scoreline.redraw |= SCORELINE_BULLETS ;
-			clear_field(&board[x_tmp][y_tmp]);
+			clear_field(x_tmp,y_tmp);
 			break;
 		case KEY:
 			robbo.keys++;
 			scoreline.redraw |= SCORELINE_KEYS ;
-			clear_field(&board[x_tmp][y_tmp]);     
+			clear_field(x_tmp,y_tmp);     
 			break;
 		case DOOR:				/* Robbo cannot move */
 			if(robbo.keys > 0){	/* should open the door first */
 				robbo.keys--;
 				scoreline.redraw |= SCORELINE_KEYS ;
-				clear_field(&board[x_tmp][y_tmp]);
+				clear_field(x_tmp,y_tmp);
 				robbo.moved = DELAY_ROBBO;
 			}
 			return;
@@ -1443,18 +1440,27 @@ void move_robbo(int x,int y)
 				/* Check for pack completion */
 				if (level_packs[selected_pack].level_selected >= level_packs[selected_pack].last_level) {
 					game_mode = END_SCREEN;
-					show_endscreen();
+					/* show_endscreen(); Redundant */
 					return;
 				} else {
 					level_packs[selected_pack].level_selected++;
-					if (level_packs[selected_pack].level_selected > level_packs[selected_pack].level_reached) level_packs[selected_pack].level_reached = level_packs[selected_pack].level_selected;
+					if (level_packs[selected_pack].level_selected > level_packs[selected_pack].level_reached) {
+						level_packs[selected_pack].level_reached = level_packs[selected_pack].level_selected;
+						/* Save rcfile now if requested -- recommended for battery operated or unstable devices */
+						if (rcfile.save_frequency == RCFILE_SAVE_ON_CHANGE) {
+							save_resource_file(path_resource_file, FALSE);
+							#if defined(PLATFORM_GP2X)
+								system ("sync");
+							#endif
+						}
+					}
 					if (level_init()) {
 						game_mode = INTRO_SCREEN;
-						show_introscreen(REDRAW_INITIALISE);
+						intro_screen.redraw |= REDRAW_INITIALISE;
 					} else {
-						show_game_area(REDRAW_INITIALISE);
+						game_area.redraw |= REDRAW_EVERYTHING;
 						/* Initialise the fade */
-						show_game_area_fade(REDRAW_INITIALISE, 16);
+						show_game_area_fade(FADE_SUB_INITIALISE, 16);
 					}
 					return;
 				}
@@ -1649,17 +1655,21 @@ void redraw_field(int x, int y) {
 /***************************************************************************
  * Field clearing                                                          *
  ***************************************************************************/
-/* Clear everything */
+/* This creates a blowable EMPTY_FIELD. This function clears id_questionmark
+   whereas create_object() doesn't */
 
-void clear_field(struct object *obj)
+void clear_field(int x, int y)
 {
-	int count;
+	create_object(x, y, EMPTY_FIELD);
+	board[x][y].id_questionmark = 0;
 
-	obj->type = 0;
+/*	int count; Redundant.
+
+	obj->type = EMPTY_FIELD;
 	obj->state = 0;
 	obj->direction = 0;
 	obj->destroyable = 0;
-	obj->blowable = 0;
+	obj->blowable = 1;
 	obj->killing = 0;
 	obj->moved = 0;
 	obj->blowed = 0;
@@ -1677,11 +1687,11 @@ void clear_field(struct object *obj)
 	obj->shooting = 0;
 	obj->processed = cycle_count;
 	obj->redraw = TRUE;
-	/* Reset the icon offsets within the image file */
+	 Reset the icon offsets within the image file 
 	for(count = 0; count < MAX_ICONS; count++) {
 		obj->icon[count].x = 0;
 		obj->icon[count].y = 0;
-	}
+	}*/
 }
 
 /***************************************************************************
@@ -1792,6 +1802,7 @@ void create_object(int x, int y, int type)
 		case LASER_D:
 		case TELEPORT:
 		case GUN:
+		case EMPTY_FIELD:
 			board[x][y].blowable = 1;			/* Via a bomb */
 			break;
 	}
@@ -1808,10 +1819,10 @@ void create_object(int x, int y, int type)
 void clear_entire_board(void) {
 	int xpos, ypos;
 
-	/* Zeroise the game board */
+	/* Fill the game board with EMPTY_FIELD objects */
 	for (ypos = 0; ypos < MAX_H; ypos++) {
 		for (xpos = 0; xpos < MAX_W; xpos++) {
-			clear_field(&board[xpos][ypos]);
+			clear_field(xpos, ypos);
 		}
 	}
 }
