@@ -28,7 +28,9 @@
 /* Defines */
 /*
 #define DEBUG_CONTROLS
+#define DEBUG_SDL_ACTIVEEVENT
 */
+
 #define JOYDEADZONE (32767 * joystick_dead_zone / 100)
 
 /* GP2X button IDs */
@@ -69,13 +71,205 @@
 /* Variables */
 SDL_Event event;
 
+#ifdef HILDON
+// #include <X11/Xutil.h>
+#include "SDL/SDL_syswm.h"
+#include "touchscreen.h"
+unsigned char touchKeysPressed[TK_COUNT];
+
+int handleMouseEvent(SDL_Event * event, int *device, int *id, int *state) {
+	int hit = 0;
+  int fstate = 0;
+
+	switch(event->type) {
+		case SDL_MOUSEBUTTONDOWN:
+
+			if(event->button.x > (KEY_UP_X - KEYRADIUS) && event->button.x < (KEY_UP_X + KEYRADIUS)
+					&& event->button.y > (KEY_UP_Y - KEYRADIUS) && event->button.y < (KEY_UP_Y + KEYRADIUS)) {
+				// Up
+				hit = 1;
+				event->key.keysym.sym = SDLK_UP;
+				event->type = SDL_KEYDOWN;
+				touchKeysPressed[TK_UP] = 1;
+        fstate = SDL_PRESSED;
+			} else if(event->button.x > (KEY_DOWN_X - KEYRADIUS) && event->button.x < (KEY_DOWN_X + KEYRADIUS)
+					&& event->button.y > (KEY_DOWN_Y - KEYRADIUS) && event->button.y < (KEY_DOWN_Y + KEYRADIUS)) {
+				// Down
+				hit = 1;
+				event->key.keysym.sym = SDLK_DOWN;
+				event->type = SDL_KEYDOWN;
+				touchKeysPressed[TK_DOWN] = 1;
+        fstate = SDL_PRESSED;
+			} else if(event->button.x > (KEY_LEFT_X - KEYRADIUS) && event->button.x < (KEY_LEFT_X + KEYRADIUS)
+					&& event->button.y > (KEY_LEFT_Y - KEYRADIUS) && event->button.y < (KEY_LEFT_Y + KEYRADIUS)) {
+				// Left
+				hit = 1;
+				event->key.keysym.sym = SDLK_LEFT;
+				event->type = SDL_KEYDOWN;
+				touchKeysPressed[TK_LEFT] = 1;
+        fstate = SDL_PRESSED;
+			} else if(event->button.x > (KEY_RIGHT_X - KEYRADIUS) && event->button.x < (KEY_RIGHT_X + KEYRADIUS)
+					&& event->button.y > (KEY_RIGHT_Y - KEYRADIUS) && event->button.y < (KEY_RIGHT_Y + KEYRADIUS)) {
+				// Right
+				hit = 1;
+				event->key.keysym.sym = SDLK_RIGHT;
+				event->type = SDL_KEYDOWN;
+				touchKeysPressed[TK_RIGHT] = 1;
+        fstate = SDL_PRESSED;
+			}
+
+			break;
+
+		case SDL_MOUSEBUTTONUP:
+
+			// Check Up
+			if(touchKeysPressed[TK_UP]) {
+				hit = 1;
+				event->key.keysym.sym = SDLK_UP;
+				event->type = SDL_KEYUP;
+				touchKeysPressed[TK_UP] = 0;
+        fstate = SDL_RELEASED;
+			}
+
+			// Check Down
+			if(touchKeysPressed[TK_DOWN]) {
+				hit = 1;
+				event->key.keysym.sym = SDLK_DOWN;
+				event->type = SDL_KEYUP;
+				touchKeysPressed[TK_DOWN] = 0;
+        fstate = SDL_RELEASED;
+			}
+
+			// Check Left
+			if(touchKeysPressed[TK_LEFT]) {
+				hit = 1;
+				event->key.keysym.sym = SDLK_LEFT;
+				event->type = SDL_KEYUP;
+				touchKeysPressed[TK_LEFT] = 0;
+        fstate = SDL_RELEASED;
+			}
+
+			// Check Right
+			if(touchKeysPressed[TK_RIGHT]) {
+				hit = 1;
+				event->key.keysym.sym = SDLK_RIGHT;
+				event->type = SDL_KEYUP;
+				touchKeysPressed[TK_RIGHT] = 0;
+        fstate = SDL_RELEASED;
+			}
+
+			break;
+	}
+
+	if(hit) {
+    *device = DEVICE_KEYBOARD;
+		*id = event->key.keysym.sym;
+		*state = fstate;
+	}
+
+	return 0;
+}
+#endif
+
+unsigned char joyHatState[10];
+
+int handleJoystickHat(SDL_Event * event, int *device, int *id, int *state) {
+#ifdef DEBUG_CONTROLS  
+  printf("SDL_JOYHATEVENT: 0x%02X\n", event->jhat.value);
+#endif  
+
+  /* simulate cursor keys */
+  if((event->jhat.value & SDL_HAT_LEFTUP) == SDL_HAT_LEFTUP) {
+    *state = SDL_PRESSED;
+    *id = user_controls[ACTION_UP_LEFT].id;
+  } else if((event->jhat.value & SDL_HAT_RIGHTUP) == SDL_HAT_RIGHTUP) {
+    *state = SDL_PRESSED;
+    *id = user_controls[ACTION_UP_RIGHT].id;
+  } else if((event->jhat.value & SDL_HAT_LEFTDOWN) == SDL_HAT_LEFTDOWN) {
+    *state = SDL_PRESSED;
+    *id = user_controls[ACTION_DOWN_LEFT].id;
+  } else if((event->jhat.value & SDL_HAT_RIGHTDOWN) == SDL_HAT_RIGHTDOWN) {
+    *state = SDL_PRESSED;
+    *id = user_controls[ACTION_DOWN_RIGHT].id;
+  } else if((event->jhat.value & SDL_HAT_UP) == SDL_HAT_UP) {
+    *state = SDL_PRESSED;
+    *id = SDLK_UP; //user_controls[ACTION_UP].id;
+  } else if((event->jhat.value & SDL_HAT_DOWN) == SDL_HAT_DOWN) {
+    *state = SDL_PRESSED;
+    *id = SDLK_DOWN; //user_controls[ACTION_DOWN].id;
+  } else if((event->jhat.value & SDL_HAT_LEFT) == SDL_HAT_LEFT) {
+    *state = SDL_PRESSED;
+    *id = SDLK_LEFT; //user_controls[ACTION_LEFT].id;
+  } else if((event->jhat.value & SDL_HAT_RIGHT) == SDL_HAT_RIGHT) {
+    *state = SDL_PRESSED;
+    *id = SDLK_RIGHT; //user_controls[ACTION_RIGHT].id;
+  } else {
+    if((joyHatState[event->jhat.hat] & SDL_HAT_LEFTUP) == SDL_HAT_LEFTUP) {
+      *id = user_controls[ACTION_UP_LEFT].id;
+    } else if((joyHatState[event->jhat.hat] & SDL_HAT_RIGHTUP) == SDL_HAT_RIGHTUP) {
+      *id = user_controls[ACTION_UP_RIGHT].id;
+    } else if((joyHatState[event->jhat.hat] & SDL_HAT_LEFTDOWN) == SDL_HAT_LEFTDOWN) {
+      *id = user_controls[ACTION_DOWN_LEFT].id;
+    } else if((joyHatState[event->jhat.hat] & SDL_HAT_RIGHTDOWN) == SDL_HAT_RIGHTDOWN) {
+      *id = user_controls[ACTION_DOWN_RIGHT].id;
+    } else if((joyHatState[event->jhat.hat] & SDL_HAT_UP) == SDL_HAT_UP) {
+      *id = SDLK_UP; //user_controls[ACTION_UP].id;
+    } else if((joyHatState[event->jhat.hat] & SDL_HAT_DOWN) == SDL_HAT_DOWN) {
+      *id = SDLK_DOWN; //user_controls[ACTION_DOWN].id;
+    } else if((joyHatState[event->jhat.hat] & SDL_HAT_LEFT) == SDL_HAT_LEFT) {
+      *id = SDLK_LEFT; //user_controls[ACTION_LEFT].id;
+    } else if((joyHatState[event->jhat.hat] & SDL_HAT_RIGHT) == SDL_HAT_RIGHT) {
+      *id = SDLK_RIGHT; //user_controls[ACTION_RIGHT].id;
+    }
+    *state = SDL_RELEASED;
+  }
+  joyHatState[event->jhat.hat] = event->jhat.value;
+
+  /* we simulate key strokes */
+  if(*id != UNDEFINED)
+    *device = DEVICE_KEYBOARD;
+  else
+    *device = DEVICE_JOYSTICK;
+
+  /* add as new options to choose */
+  /*
+     if((event.jhat.value & SDL_HAT_UP) == SDL_HAT_UP) {
+   *state = SDL_PRESSED;
+   *id = 0;
+   } else if((event.jhat.value & SDL_HAT_DOWN) == SDL_HAT_DOWN) {
+   *state = SDL_PRESSED;
+   *id = 1;
+   } else if((event.jhat.value & SDL_HAT_LEFT) == SDL_HAT_LEFT) {
+   *state = SDL_PRESSED;
+   *id = 2;
+   } else if((event.jhat.value & SDL_HAT_RIGHT) == SDL_HAT_RIGHT) {
+   *state = SDL_PRESSED;
+   *id = 3;
+   } else if((joyHatState[event.jhat.hat] & SDL_HAT_UP) == SDL_HAT_UP) {
+   *state = SDL_RELEASED;
+   *id = 0;
+   } else if((joyHatState[event.jhat.hat] & SDL_HAT_DOWN) == SDL_HAT_DOWN) {
+   *state = SDL_RELEASED;
+   *id = 1;
+   } else if((joyHatState[event.jhat.hat] & SDL_HAT_LEFT) == SDL_HAT_LEFT) {
+   *state = SDL_RELEASED;
+   *id = 2;
+   } else if((joyHatState[event.jhat.hat] & SDL_HAT_RIGHT) == SDL_HAT_RIGHT) {
+   *state = SDL_RELEASED;
+   *id = 3;
+   }
+   joyHatState[event.jhat.hat] = event.jhat.value;
+   *id *= event.jhat.hat + 1;
+   *id += SDL_JoystickNumAxes(joystick)*2 + SDL_JoystickNumButtons(joystick);
+   */
+  return 0;
+}
+
 /* Function prototypes */
-void show_user_action(int *action_found);
-int set_key_repeat(int delay, int interval);
 
 
 /***************************************************************************
- * Get User Action v5                                                      *
+ * Get User Action v5.3                                                    *
  ***************************************************************************/
 /* This reports whether an action has been triggered via user input.
    Controls with modifiers take precedence over single key controls.
@@ -87,16 +281,30 @@ int set_key_repeat(int delay, int interval);
    operation you should only need to know the action triggered.
    This function manages key/button repeat which can be different for each
    control. If you don't want key/button repeat then set delay to -1.
-   It's designed to be called every TICK_INTERVAL because it counts in
-   TICK_INTERVAL units for key/button repeat calculation.
+   It's designed to be called every game cycle because it counts in
+   game cycle units for key/button repeat calculation.
    The event loop code in END SCREEN in main is almost empty and makes a good
-   template. Enable DEBUG to see the ACTIONs firing.
+   template. Enable DEBUG_CONTROLS to see the ACTIONs firing.
    
    UPDATE: Joystick axis support added. Axes are converted into additional
    virtual buttons (two per axis) and require a dead zone to be set up which
    should be user configurable. For example, if a joystick has 10 physical
    buttons then axis0 will be virtual buttons 10 and 11, axis1 12 and 13 etc.   
 
+   v5.1 UPDATE: pollall now breaks when an action is triggered to prevent the
+   possibility that an action is pressed and released before exiting (as
+   experienced on my Zaurus).
+
+   v5.2 UPDATE: Added mouse support. Initially I was intending to substitute
+   this version with another from an event/object based SDL engine I've been
+   working on but it won't work. In the end adding mouse/stylus support was
+   easier than I was expecting. I've added a ACTION_PRIMARY_CLICK action that
+   can be set-up for use with a mouse or stylus.
+
+   v5.3 UPDATE: Added hooks to the GNU Robbo Rectangular Object Engine (ROB).
+   ROB likes to be informed of released and triggered actions so that it can
+   generate events and manage the simulated pointer via the cursor events.
+
    struct control {
    int device;			Keyboard or joystick 
    int id;				Key or button id 
@@ -109,16 +317,16 @@ int set_key_repeat(int delay, int interval);
    struct control user_controls[USER_CONTROLS];		Holds the assignment of user controls 
    
    On entry: actionid will hold any actionid found
-			 pollall = TRUE to poll all events
+			 pollall = TRUE to poll all events until an action is triggered
 			 pollall = FALSE to poll one event (used to retrieve the device, id and state for each polled event)
 			 device will hold the device of the last event or UNDEFINED (-1)
 			 id will hold the id of the last event or UNDEFINED
 			 state will hold the state of the last event or UNDEFINED
-   On exit:  actionid will contain an actionid or UNDEFINED
+	On exit: actionid will contain an actionid or UNDEFINED
 			 returns 1 on SDL_QUIT else 0 */   
 
 int get_user_action(int *actionid, int pollall, int *device, int *id, int *state) {
-	int count, found = FALSE, axisplus, quitfound = 0;
+	int count, found = FALSE, axisplus, quitfound = 0, rob_actionid;
 	static int axisvalues[10 * 2] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};	/* I doubt there exists a joystick with more than 10 axes */
 	
 	*actionid = UNDEFINED;
@@ -132,12 +340,21 @@ int get_user_action(int *actionid, int pollall, int *device, int *id, int *state
 				*id = event.key.keysym.sym;
 				*state = event.key.state;
 				break;
+#ifdef HHILDON
+			case SDL_MOUSEBUTTONDOWN:
+			case SDL_MOUSEBUTTONUP:
+        handleMouseEvent(&event, device, id, state);
+        break;
+#endif
 			case SDL_JOYBUTTONUP:
 			case SDL_JOYBUTTONDOWN:
 				*device = DEVICE_JOYSTICK;
 				*id = event.jbutton.button;
 				*state = event.jbutton.state;
 				break;
+			case SDL_JOYHATMOTION:
+        handleJoystickHat(&event, device, id, state);
+        break;
 			case SDL_JOYAXISMOTION:
 				/* Identify which end of the axis we are going to work with */
 				axisplus = 0;	/* -32767 to 0 */
@@ -150,21 +367,77 @@ int get_user_action(int *actionid, int pollall, int *device, int *id, int *state
 					axisvalues[event.jaxis.axis * 2 + axisplus] = abs(event.jaxis.value);
 				}
 				break;
+#ifdef HHILDON
+      case SDL_SYSWMEVENT:
+        printf("Xevent: %d:%d\n", event.syswm.msg->event.xevent.type, event.syswm.msg->event.xevent.xclient.format);
+        printf("  data: ");
+        { 
+          int ii;
+          for(ii=0;ii<20;ii++)
+            printf("%02X", event.syswm.msg->event.xevent.xclient.data.b[ii]);
+          printf("\n");
+        }
+        printf("SDL_GetAppState: 0x%02X\n", SDL_GetAppState());
+        // name = XGetAtomName(dpy, property);
+        // printf("\t%s (%ld):\t", name, property);
+        // printf("  message_type: %ld\n", event.syswm.msg->event.xevent.xclient.message_type);
+        // if(event.syswm.msg->event.xevent.type == 33 && event.syswm.msg->event.xevent.xclient.message_type == 327
+        break;
+
+      case SDL_ACTIVEEVENT:
+        printf("SDL_ACTIVEEVENT gain:0x%02X, state:0x%02X\n", event.active.gain, event.active.state);
+        if (event.active.state & SDL_APPMOUSEFOCUS) printf("SDL_APPMOUSEFOCUS ");
+        if (event.active.state & SDL_APPINPUTFOCUS) printf("SDL_APPINPUTFOCUS ");
+        if (event.active.state & SDL_APPACTIVE) printf("SDL_APPACTIVE ");
+        break;
+#endif
+			case SDL_MOUSEBUTTONUP:
+			case SDL_MOUSEBUTTONDOWN:
+				*device = DEVICE_MOUSE;
+				*id = event.button.button;
+				*state = event.button.state;
+				break;
+			case SDL_MOUSEMOTION:
+			 	rob_lyr_pointer->x = event.motion.x;
+			 	rob_lyr_pointer->y = event.motion.y;
+				break;
 			case SDL_QUIT:
 				quitfound = 1;
 				break;
+		#ifdef DEBUG_SDL_ACTIVEEVENT
+			case SDL_ACTIVEEVENT:
+				printf("%i: states: ", cycle_count);
+				if (event.active.state & SDL_APPMOUSEFOCUS) printf("SDL_APPMOUSEFOCUS ");
+				if (event.active.state & SDL_APPINPUTFOCUS) printf("SDL_APPINPUTFOCUS ");
+				if (event.active.state & SDL_APPACTIVE) printf("SDL_APPACTIVE ");
+				printf(": gain=%i\n", event.active.gain);
+				break;
+		#endif
 			default:
+				// handleWindowEvent(&event);
 				break;
 		}
 
 		/* Iterate through user_controls and record the state for all matching controls */
+		found = FALSE;
 		if (*device != UNDEFINED) {
+			/* Iterate through user_controls and record the state for all matching controls */
 			#ifdef DEBUG_CONTROLS
 				printf("device=%i; id=%03i; state=%i; all control states:", *device, *id, *state);
 			#endif
 			for (count = 0; count < USER_CONTROLS; count++) {
-				if (user_controls[count].device == *device && user_controls[count].id == *id) user_controls[count].state = *state;
-				if (*state == SDL_RELEASED) user_controls[count].cyclesactive = 0;
+				if (user_controls[count].device == *device && user_controls[count].id == *id) {
+					user_controls[count].state = *state;
+					if (*state == SDL_RELEASED) {
+						user_controls[count].cyclesactive = 0;
+						/* ROB will want to know what action was released. Actually this isn't reporting
+						   action releases but control releases which is not the same thing. The reporting
+						   of action releases was not part of the requirements when designing this version
+						   of get_user_action() for GNU Robbo, but what is done here more than suffices */
+						rob_actionid = count | 0x80; ROB_GenerateEvents(&rob_actionid);
+					}
+					found = TRUE;
+				}
 				#ifdef DEBUG_CONTROLS
 					printf(" %i", user_controls[count].state);
 				#endif
@@ -172,14 +445,15 @@ int get_user_action(int *actionid, int pollall, int *device, int *id, int *state
 			#ifdef DEBUG_CONTROLS
 				printf("\n");
 			#endif
+	
+			if (found || !pollall) break;	/* Stop polling events now as we've got something to work with */
 		}
 		
-		if (!pollall) break;
-		
 	}
 
 	/* Now we know what's pressed we can see if an action has been triggered */
 	/* Controls with modifiers take precedence so search for these first */
+	found = FALSE;
 	for (count = 0; count < USER_CONTROLS; count++) {
 		if (user_controls[count].state == SDL_PRESSED && user_controls[count].mod != UNDEFINED && user_controls[user_controls[count].mod].state == SDL_PRESSED) {
 			found = TRUE;
@@ -204,8 +478,8 @@ int get_user_action(int *actionid, int pollall, int *device, int *id, int *state
 		}
 	}
 	
-	/* Because this function is called every TICK_INTERVAL, if the user is pressing a
-	   control then it would trigger an action every TICK_INTERVAL. cyclesactive, delay
+	/* Because this function is called every game cycle, if the user is pressing a
+	   control then it would trigger an action every game cycle. cyclesactive, delay
 	   and interval are used to decide the frequency of individual key/button repeats */
 	if (found) {
 		if (user_controls[*actionid].cyclesactive == 0) {
@@ -227,6 +501,9 @@ int get_user_action(int *actionid, int pollall, int *device, int *id, int *state
 		if (!found) *actionid = UNDEFINED;
 	}
 
+	/* ROB will want to know what action was triggered */
+	ROB_GenerateEvents(actionid);
+
 	return quitfound;
 }
 
@@ -240,14 +517,26 @@ void show_user_action(int *actionid) {
 		case ACTION_UP:
 			printf("ACTION_UP");
 			break;
+		case ACTION_UP_RIGHT:
+			printf("ACTION_UP_RIGHT");
+			break;
+		case ACTION_RIGHT:
+			printf("ACTION_RIGHT");
+			break;
+		case ACTION_DOWN_RIGHT:
+			printf("ACTION_DOWN_RIGHT");
+			break;
 		case ACTION_DOWN:
 			printf("ACTION_DOWN");
 			break;
+		case ACTION_DOWN_LEFT:
+			printf("ACTION_DOWN_LEFT");
+			break;
 		case ACTION_LEFT:
 			printf("ACTION_LEFT");
 			break;
-		case ACTION_RIGHT:
-			printf("ACTION_RIGHT");
+		case ACTION_UP_LEFT:
+			printf("ACTION_UP_LEFT");
 			break;
 		case ACTION_SHOOT_UP:
 			printf("ACTION_SHOOT_UP");
@@ -288,12 +577,6 @@ void show_user_action(int *actionid) {
 		case ACTION_NEXT_PACK:
 			printf("ACTION_NEXT_PACK");
 			break;
-		case ACTION_MODIFIER1:
-			printf("ACTION_MODIFIER1");
-			break;
-		case ACTION_MODIFIER2:
-			printf("ACTION_MODIFIER2");
-			break;
 		case ACTION_TOGGLE_FULLSCREEN:
 			printf("ACTION_TOGGLE_FULLSCREEN");
 			break;
@@ -315,9 +598,27 @@ void show_user_action(int *actionid) {
 		case ACTION_VOLDOWN:
 			printf("ACTION_VOLDOWN");
 			break;
+		case ACTION_MODIFIER1:
+			printf("ACTION_MODIFIER1");
+			break;
+		case ACTION_MODIFIER2:
+			printf("ACTION_MODIFIER2");
+			break;
 		case ACTION_NOT_USED1:
 			printf("ACTION_NOT_USED1");
 			break;
+		case ACTION_TOGGLE_DESIGNER:
+			printf("ACTION_TOGGLE_DESIGNER");
+			break;
+		case ACTION_SCROLL_UP:
+			printf("ACTION_SCROLL_UP");
+			break;
+		case ACTION_SCROLL_DOWN:
+			printf("ACTION_SCROLL_DOWN");
+			break;
+		case ACTION_PRIMARY_CLICK:
+			printf("ACTION_PRIMARY_CLICK");
+			break;
 		case UNDEFINED:
 			printf("UNDEFINED");
 			break;
@@ -355,9 +656,14 @@ void set_default_user_controls(struct control user_controls[USER_CONTROLS]) {
 	/* Now tune to a platform */
 	#if defined(PLATFORM_GP2X)
 		user_controls[ACTION_UP].id = GP2X_JOY_N;
-		user_controls[ACTION_DOWN].id = GP2X_JOY_S;
-		user_controls[ACTION_LEFT].id = GP2X_JOY_W;
+		user_controls[ACTION_UP_RIGHT].id = GP2X_JOY_NE;
 		user_controls[ACTION_RIGHT].id = GP2X_JOY_E;
+		user_controls[ACTION_DOWN_RIGHT].id = GP2X_JOY_SE;
+		user_controls[ACTION_DOWN].id = GP2X_JOY_S;
+		user_controls[ACTION_DOWN_LEFT].id = GP2X_JOY_SW;
+		user_controls[ACTION_LEFT].id = GP2X_JOY_W;
+		user_controls[ACTION_UP_LEFT].id = GP2X_JOY_NW;
+		user_controls[ACTION_RESTART].id = GP2X_SELECT;
 		user_controls[ACTION_SHOOT_UP].id = GP2X_JOY_N;
 		user_controls[ACTION_SHOOT_UP].mod = ACTION_MODIFIER1;
 		user_controls[ACTION_SHOOT_DOWN].id = GP2X_JOY_S;
@@ -371,7 +677,7 @@ void set_default_user_controls(struct control user_controls[USER_CONTROLS]) {
 		user_controls[ACTION_HELP].id = GP2X_BTN_Y;
 		user_controls[ACTION_HELP].mod = ACTION_MODIFIER1;
 		user_controls[ACTION_OPTIONS].id = GP2X_START;
-		user_controls[ACTION_RESTART].id = GP2X_SELECT;
+		user_controls[ACTION_TOGGLE_FULLSCREEN].device = UNDEFINED;
 		user_controls[ACTION_PREVIOUS_LEVEL].id = GP2X_LTRIG;
 		user_controls[ACTION_PREVIOUS_LEVEL].mod = ACTION_MODIFIER1;
 		user_controls[ACTION_NEXT_LEVEL].id = GP2X_RTRIG;
@@ -380,23 +686,37 @@ void set_default_user_controls(struct control user_controls[USER_CONTROLS]) {
 		user_controls[ACTION_PREVIOUS_PACK].mod = ACTION_MODIFIER2;
 		user_controls[ACTION_NEXT_PACK].id = GP2X_RTRIG;
 		user_controls[ACTION_NEXT_PACK].mod = ACTION_MODIFIER2;
-		user_controls[ACTION_MODIFIER1].id = GP2X_BTN_A;
-		user_controls[ACTION_MODIFIER2].id = GP2X_BTN_Y;
-		user_controls[ACTION_TOGGLE_FULLSCREEN].device = UNDEFINED;
 		user_controls[ACTION_HOME].id = GP2X_VOL_DN;
 		user_controls[ACTION_HOME].mod = ACTION_MODIFIER1;
 		user_controls[ACTION_END].id = GP2X_VOL_UP;
 		user_controls[ACTION_END].mod = ACTION_MODIFIER1;
 		user_controls[ACTION_PAGEUP].id = GP2X_LTRIG;
 		user_controls[ACTION_PAGEDOWN].id = GP2X_RTRIG;
+		user_controls[ACTION_TOGGLE_DESIGNER].device = UNDEFINED;
 		user_controls[ACTION_VOLUP].id = GP2X_VOL_UP;
 		user_controls[ACTION_VOLDOWN].id = GP2X_VOL_DN;
+		user_controls[ACTION_MODIFIER1].id = GP2X_BTN_A;
+		user_controls[ACTION_MODIFIER2].id = GP2X_BTN_Y;
+		user_controls[ACTION_MODIFIER3].device = UNDEFINED;
+		user_controls[ACTION_MODIFIER4].device = UNDEFINED;
+		user_controls[ACTION_SCROLL_UP].device = UNDEFINED;
+		user_controls[ACTION_SCROLL_DOWN].device = UNDEFINED;
+		user_controls[ACTION_PRIMARY_CLICK].device = UNDEFINED;
 		user_controls[ACTION_NOT_USED1].device = UNDEFINED;
 	#elif defined(PLATFORM_ZAURUS)
 		user_controls[ACTION_UP].id = ZAURUS_UP;
-		user_controls[ACTION_DOWN].id = ZAURUS_DOWN;
-		user_controls[ACTION_LEFT].id = ZAURUS_LEFT;
+		user_controls[ACTION_UP_RIGHT].id = ZAURUS_UP;
+		user_controls[ACTION_UP_RIGHT].mod = ACTION_MODIFIER3;
 		user_controls[ACTION_RIGHT].id = ZAURUS_RIGHT;
+		user_controls[ACTION_DOWN_RIGHT].id = ZAURUS_DOWN;
+		user_controls[ACTION_DOWN_RIGHT].mod = ACTION_MODIFIER3;
+		user_controls[ACTION_DOWN].id = ZAURUS_DOWN;
+		user_controls[ACTION_DOWN_LEFT].id = ZAURUS_DOWN;
+		user_controls[ACTION_DOWN_LEFT].mod = ACTION_MODIFIER4;
+		user_controls[ACTION_LEFT].id = ZAURUS_LEFT;
+		user_controls[ACTION_UP_LEFT].id = ZAURUS_UP;
+		user_controls[ACTION_UP_LEFT].mod = ACTION_MODIFIER4;
+		user_controls[ACTION_RESTART].id = ZAURUS_HOME;
 		user_controls[ACTION_SHOOT_UP].id = ZAURUS_UP;
 		user_controls[ACTION_SHOOT_UP].mod = ACTION_MODIFIER1;
 		user_controls[ACTION_SHOOT_DOWN].id = ZAURUS_DOWN;
@@ -405,30 +725,47 @@ void set_default_user_controls(struct control user_controls[USER_CONTROLS]) {
 		user_controls[ACTION_SHOOT_LEFT].mod = ACTION_MODIFIER1;
 		user_controls[ACTION_SHOOT_RIGHT].id = ZAURUS_RIGHT;
 		user_controls[ACTION_SHOOT_RIGHT].mod = ACTION_MODIFIER1;
-		user_controls[ACTION_SELECT].id = ZAURUS_OK;
+		user_controls[ACTION_SELECT].id = ZAURUS_SPACE;
 		user_controls[ACTION_EXIT].id = ZAURUS_CANCEL;
-		user_controls[ACTION_HELP].id = UNDEFINED;
-		user_controls[ACTION_OPTIONS].id = ZAURUS_ADDRESS;
-		user_controls[ACTION_RESTART].id = ZAURUS_HOME;
-		user_controls[ACTION_PREVIOUS_LEVEL].id = UNDEFINED;
-		user_controls[ACTION_NEXT_LEVEL].id = UNDEFINED;
-		user_controls[ACTION_PREVIOUS_PACK].id = UNDEFINED;
-		user_controls[ACTION_NEXT_PACK].id = UNDEFINED;
+		user_controls[ACTION_HELP].id = ZAURUS_MENU;
+		user_controls[ACTION_OPTIONS].id = ZAURUS_EMAIL;
+		user_controls[ACTION_TOGGLE_FULLSCREEN].device = UNDEFINED;
+		user_controls[ACTION_PREVIOUS_LEVEL].id = ZAURUS_CALENDAR;
+		user_controls[ACTION_PREVIOUS_LEVEL].mod = ACTION_MODIFIER1;
+		user_controls[ACTION_NEXT_LEVEL].id = ZAURUS_ADDRESS;
+		user_controls[ACTION_NEXT_LEVEL].mod = ACTION_MODIFIER1;
+		user_controls[ACTION_PREVIOUS_PACK].device = UNDEFINED;
+		user_controls[ACTION_NEXT_PACK].device = UNDEFINED;
+		user_controls[ACTION_HOME].device = UNDEFINED;
+		user_controls[ACTION_END].device = UNDEFINED;
+		user_controls[ACTION_PAGEUP].id = ZAURUS_CALENDAR;
+		user_controls[ACTION_PAGEDOWN].id = ZAURUS_ADDRESS;
+		user_controls[ACTION_TOGGLE_DESIGNER].device = UNDEFINED;
+		user_controls[ACTION_VOLUP].device = UNDEFINED;
+		user_controls[ACTION_VOLDOWN].device = UNDEFINED;
 		user_controls[ACTION_MODIFIER1].id = ZAURUS_SPACE;
-		user_controls[ACTION_MODIFIER2].id = UNDEFINED;
-		user_controls[ACTION_TOGGLE_FULLSCREEN].id = UNDEFINED;
-		user_controls[ACTION_HOME].id = UNDEFINED;
-		user_controls[ACTION_END].id = UNDEFINED;
-		user_controls[ACTION_PAGEUP].id = UNDEFINED;
-		user_controls[ACTION_PAGEDOWN].id = UNDEFINED;
-		user_controls[ACTION_VOLUP].id = UNDEFINED;
-		user_controls[ACTION_VOLDOWN].id = UNDEFINED;
+		user_controls[ACTION_MODIFIER2].device = UNDEFINED;
+		user_controls[ACTION_MODIFIER3].id = ZAURUS_RIGHT;
+		user_controls[ACTION_MODIFIER4].id = ZAURUS_LEFT;
+		user_controls[ACTION_SCROLL_UP].device = UNDEFINED;
+		user_controls[ACTION_SCROLL_DOWN].device = UNDEFINED;
+		user_controls[ACTION_PRIMARY_CLICK].device = DEVICE_MOUSE;
+		user_controls[ACTION_PRIMARY_CLICK].id = SDL_BUTTON_LEFT;
 		user_controls[ACTION_NOT_USED1].device = UNDEFINED;
 	#elif defined(PLATFORM_PC)
 		user_controls[ACTION_UP].id = SDLK_UP;
-		user_controls[ACTION_DOWN].id = SDLK_DOWN;
-		user_controls[ACTION_LEFT].id = SDLK_LEFT;
+		user_controls[ACTION_UP_RIGHT].id = SDLK_UP;
+		user_controls[ACTION_UP_RIGHT].mod = ACTION_MODIFIER3;
 		user_controls[ACTION_RIGHT].id = SDLK_RIGHT;
+		user_controls[ACTION_DOWN_RIGHT].id = SDLK_DOWN;
+		user_controls[ACTION_DOWN_RIGHT].mod = ACTION_MODIFIER3;
+		user_controls[ACTION_DOWN].id = SDLK_DOWN;
+		user_controls[ACTION_DOWN_LEFT].id = SDLK_DOWN;
+		user_controls[ACTION_DOWN_LEFT].mod = ACTION_MODIFIER4;
+		user_controls[ACTION_LEFT].id = SDLK_LEFT;
+		user_controls[ACTION_UP_LEFT].id = SDLK_UP;
+		user_controls[ACTION_UP_LEFT].mod = ACTION_MODIFIER4;
+		user_controls[ACTION_RESTART].id = SDLK_r;
 		user_controls[ACTION_SHOOT_UP].id = SDLK_UP;
 		user_controls[ACTION_SHOOT_UP].mod = ACTION_MODIFIER2;
 		user_controls[ACTION_SHOOT_DOWN].id = SDLK_DOWN;
@@ -441,21 +778,72 @@ void set_default_user_controls(struct control user_controls[USER_CONTROLS]) {
 		user_controls[ACTION_EXIT].id = SDLK_ESCAPE;
 		user_controls[ACTION_HELP].id = SDLK_F1;
 		user_controls[ACTION_OPTIONS].id = SDLK_F4;
-		user_controls[ACTION_RESTART].id = SDLK_r;
+		user_controls[ACTION_TOGGLE_FULLSCREEN].id = SDLK_RETURN;
+		user_controls[ACTION_TOGGLE_FULLSCREEN].mod = ACTION_MODIFIER1;
 		user_controls[ACTION_PREVIOUS_LEVEL].id = SDLK_F5;
 		user_controls[ACTION_NEXT_LEVEL].id = SDLK_F6;
 		user_controls[ACTION_PREVIOUS_PACK].id = SDLK_F7;
 		user_controls[ACTION_NEXT_PACK].id = SDLK_F8;
-		user_controls[ACTION_MODIFIER1].id = SDLK_LALT;
-		user_controls[ACTION_MODIFIER2].id = SDLK_RSHIFT;
-		user_controls[ACTION_TOGGLE_FULLSCREEN].id = SDLK_RETURN;
-		user_controls[ACTION_TOGGLE_FULLSCREEN].mod = ACTION_MODIFIER1;
 		user_controls[ACTION_HOME].id = SDLK_HOME;
 		user_controls[ACTION_END].id = SDLK_END;
 		user_controls[ACTION_PAGEUP].id = SDLK_PAGEUP;
 		user_controls[ACTION_PAGEDOWN].id = SDLK_PAGEDOWN;
+		user_controls[ACTION_TOGGLE_DESIGNER].device = UNDEFINED;
 		user_controls[ACTION_VOLUP].id = SDLK_EQUALS;
 		user_controls[ACTION_VOLDOWN].id = SDLK_MINUS;
+		user_controls[ACTION_MODIFIER1].id = SDLK_LALT;
+		user_controls[ACTION_MODIFIER2].id = SDLK_RSHIFT;
+		user_controls[ACTION_MODIFIER3].id = SDLK_RIGHT;
+		user_controls[ACTION_MODIFIER4].id = SDLK_LEFT;
+		user_controls[ACTION_SCROLL_UP].device = UNDEFINED;
+		user_controls[ACTION_SCROLL_DOWN].device = UNDEFINED;
+		user_controls[ACTION_PRIMARY_CLICK].device = DEVICE_MOUSE;
+		user_controls[ACTION_PRIMARY_CLICK].id = SDL_BUTTON_LEFT;
+		user_controls[ACTION_NOT_USED1].device = UNDEFINED;
+	#elif defined(HILDON)
+		user_controls[ACTION_UP].id = SDLK_UP;
+		user_controls[ACTION_UP_RIGHT].id = SDLK_UP;
+		user_controls[ACTION_UP_RIGHT].mod = ACTION_MODIFIER3;
+		user_controls[ACTION_RIGHT].id = SDLK_RIGHT;
+		user_controls[ACTION_DOWN_RIGHT].id = SDLK_DOWN;
+		user_controls[ACTION_DOWN_RIGHT].mod = ACTION_MODIFIER3;
+		user_controls[ACTION_DOWN].id = SDLK_DOWN;
+		user_controls[ACTION_DOWN_LEFT].id = SDLK_DOWN;
+		user_controls[ACTION_DOWN_LEFT].mod = ACTION_MODIFIER4;
+		user_controls[ACTION_LEFT].id = SDLK_LEFT;
+		user_controls[ACTION_UP_LEFT].id = SDLK_UP;
+		user_controls[ACTION_UP_LEFT].mod = ACTION_MODIFIER4;
+		user_controls[ACTION_RESTART].id = SDLK_r;
+		user_controls[ACTION_SHOOT_UP].id = SDLK_UP;
+		user_controls[ACTION_SHOOT_UP].mod = ACTION_MODIFIER2;
+		user_controls[ACTION_SHOOT_DOWN].id = SDLK_DOWN;
+		user_controls[ACTION_SHOOT_DOWN].mod = ACTION_MODIFIER2;
+		user_controls[ACTION_SHOOT_LEFT].id = SDLK_LEFT;
+		user_controls[ACTION_SHOOT_LEFT].mod = ACTION_MODIFIER2;
+		user_controls[ACTION_SHOOT_RIGHT].id = SDLK_RIGHT;
+		user_controls[ACTION_SHOOT_RIGHT].mod = ACTION_MODIFIER2;
+		user_controls[ACTION_SELECT].id = SDLK_RETURN;
+		user_controls[ACTION_EXIT].id = SDLK_ESCAPE;
+		user_controls[ACTION_HELP].id = SDLK_F1;
+		user_controls[ACTION_OPTIONS].id = SDLK_F4;
+		user_controls[ACTION_TOGGLE_FULLSCREEN].id = SDLK_F6;
+		user_controls[ACTION_PREVIOUS_LEVEL].id = SDLK_F8;
+		user_controls[ACTION_NEXT_LEVEL].id = SDLK_F7;
+		user_controls[ACTION_PREVIOUS_PACK].device = UNDEFINED;
+		user_controls[ACTION_NEXT_PACK].device = UNDEFINED;
+		user_controls[ACTION_HOME].id = SDLK_HOME;
+		user_controls[ACTION_END].id = SDLK_END;
+		user_controls[ACTION_PAGEUP].id = SDLK_PAGEUP;
+		user_controls[ACTION_PAGEDOWN].id = SDLK_PAGEDOWN;
+		user_controls[ACTION_TOGGLE_DESIGNER].device = UNDEFINED;
+		user_controls[ACTION_VOLUP].device = UNDEFINED;
+		user_controls[ACTION_VOLDOWN].device = UNDEFINED;
+		user_controls[ACTION_MODIFIER1].id = SDLK_LALT;
+		user_controls[ACTION_MODIFIER2].id = SDLK_RETURN;
+		user_controls[ACTION_SCROLL_UP].device = UNDEFINED;
+		user_controls[ACTION_SCROLL_DOWN].device = UNDEFINED;
+		user_controls[ACTION_PRIMARY_CLICK].device = DEVICE_MOUSE;
+		user_controls[ACTION_PRIMARY_CLICK].id = SDL_BUTTON_LEFT;
 		user_controls[ACTION_NOT_USED1].device = UNDEFINED;
 	#endif
 
@@ -464,7 +852,7 @@ void set_default_user_controls(struct control user_controls[USER_CONTROLS]) {
 /***************************************************************************
  * Set Key Repeat                                                          *
  ***************************************************************************/
-/* Enable or disable keyboard key repeat (not used in this project anymore)
+/* Enable or disable keyboard key repeat.
 
    On entry: delay = the delay before repeating or 0 to disable
 			 interval = the repeat interval
@@ -582,6 +970,13 @@ int initialise_joystick(int joyid, char *joyname, int show) {
 		joyid = UNDEFINED;
 	}
 
+  if(joystick) {
+    fprintf(stdout,"Number of axis   : %d\n", SDL_JoystickNumAxes(joystick));
+    fprintf(stdout,"Number of balls  : %d\n", SDL_JoystickNumBalls(joystick));
+    fprintf(stdout,"Number of buttons: %d\n", SDL_JoystickNumButtons(joystick));
+    fprintf(stdout,"Number of hats   : %d\n", SDL_JoystickNumHats(joystick));
+  }
+  
 	if (joystick) while (SDL_PollEvent(&event));	/* Flush events as opening a joystick releases all the buttons */
 	get_joystick_name(joyid, joyname);
 	return joyid;
@@ -661,7 +1056,28 @@ int get_joystick_list(char joystick_list[MAX_JOYSTICKS][MAX_JOYSTICK_NAME_LENGTH
     On exit: text = either a key/button legend or the id in text */
 
 void get_keyjbtn_text(int device, int id, char *joyname, char *text) {
-	if (device == DEVICE_KEYBOARD) {
+	if (device == DEVICE_MOUSE) {
+		switch (id) {
+			case SDL_BUTTON_LEFT:
+				strcpy(text, txt_ptr_BtnLeft);
+				break;
+			case SDL_BUTTON_MIDDLE:
+				strcpy(text, txt_ptr_BtnMiddle);
+				break;
+			case SDL_BUTTON_RIGHT:
+				strcpy(text, txt_ptr_BtnRight);
+				break;
+			case SDL_BUTTON_WHEELUP:
+				strcpy(text, txt_ptr_WheelUp);
+				break;
+			case SDL_BUTTON_WHEELDOWN:
+				strcpy(text, txt_ptr_WheelDown);
+				break;
+			default:
+				sprintf(text, "%i", id);
+				break;
+		}
+	} else if (device == DEVICE_KEYBOARD) {
 		switch (id) {
 			case SDLK_UNKNOWN:
 				strcpy(text, "SDLUnknown");
@@ -1187,9 +1603,37 @@ void get_keyjbtn_text(int device, int id, char *joyname, char *text) {
 					/*strcpy(text, "Unknown");*/
 					break;
 			}
+		} else if ((strcmp(joyname, "Logitech Logitech Dual Action")) == 0) {	/* Logitech Dual Action */
+			if(id>=0 && id <=9) {
+				sprintf(text, "Button%i", id+1);
+			} else {
+        switch (id) {
+          case 12:
+          case 16:
+            strcpy(text, "Left");
+            break;
+          case 13:
+          case 17:
+            strcpy(text, "Right");
+            break;
+          case 14:
+          case 18:
+            strcpy(text, "Up");
+            break;
+          case 15:
+          case 19:
+            strcpy(text, "Down");
+            break;
+          default:
+            sprintf(text, "%i", id);
+            break;
+        }
+			}
 		} else {
 			sprintf(text, "%i", id);
 		}
+	} else {
+		sprintf(text, "%i", device);
 	}
 }
 
