@@ -25,7 +25,11 @@
 
 #include <SDL/SDL.h>
 #include <SDL/SDL_endian.h>
-#include <SDL/SDL_ttf.h>
+#ifdef USE_PIXMAP_FONT
+	#include "font.h"
+#else
+	#include <SDL/SDL_ttf.h>
+#endif
 #include <string.h>
 #include <stdlib.h>
 #include <math.h>		/* For ceil() */
@@ -33,6 +37,7 @@
 #include <sys/stat.h>	/* For time_t struct */
 #include <unistd.h>		/* For getpid() */
 #include <dirent.h>		/* For dir functions */
+#include "ROB_engine.h"
 #include "screen.h"
 #include "board.h"
 #include "controls.h"
@@ -41,6 +46,7 @@
 #include "rcfile.h"
 #include "sound.h"
 #include "locales.h"
+#include "pointer_controls.h"
 
 /* Defines */
 /*
@@ -71,14 +77,51 @@
 #define HELP_SCREEN 3
 #define OPTIONS_SCREEN 4
 
-#define HELP_SCREEN_PAGES 4
-#define OPTIONS_SCREEN_PAGES 6
+/* Hotspot ids */
+#define HOTSPOTS_ID 100		/* Requires at most OPTIONS_COUNT * 3 unique ids */
 
-#define OPTIONS_MENUPOS_LEFT 9
+/* Help and Options page counts */
+#define HELP_SCREEN_PAGES 4
+#define OPTIONS_SCREEN_PAGES 8
+
+/* Option page locations */
+#define OPTIONS_MENUPOS_LEFT 9	
 #define OPTIONS_MENUPOS_SAVE 10
 #define OPTIONS_MENUPOS_EXIT 11
 #define OPTIONS_MENUPOS_RIGHT 12
 
+/* Option count */
+#define OPTIONS_COUNT 104
+
+/* Option ids */
+#define OPTIONS_GAME_SPEED (0 * 13 + 0)
+#define OPTIONS_DEFAULT_JOYSTICK (0 * 13 + 1)
+#define OPTIONS_JOYSTICK_AXES_DEAD_ZONE (0 * 13 + 2)
+#define OPTIONS_KEY_REPEAT_DELAY (0 * 13 + 4)
+#define OPTIONS_KEY_REPEAT_INTERVAL (0 * 13 + 5)
+#define OPTIONS_LANGUAGE (0 * 13 + 6)
+#define OPTIONS_SAVE_FREQUENCY (0 * 13 + 8)
+#define OPTIONS_RECONFIGURATION_HELP (1 * 13 + 0)
+#define OPTIONS_ACTION_UP (2 * 13 + 0)
+#define OPTIONS_ACTION_MODIFIER1 (5 * 13 + 2)
+#define OPTIONS_ACTION_MODIFIER4 (5 * 13 + 5)
+#define OPTIONS_ACTION_PRIMARY_CLICK (5 * 13 + 8)
+#define OPTIONS_RESTORE_DEFAULT_CONTROLS (6 * 13 + 0)
+#define OPTIONS_SYSTEM_POINTER (6 * 13 + 1)
+#define OPTIONS_SKIN (7 * 13 + 0)
+
+/* Intro screen menu item count */
+#define INTRO_SCREEN_MENU_ITEM_COUNT 7
+
+/* Intro screen menu item ids */
+#define INTRO_SCREEN_MENU_ITEM_START 0
+#define INTRO_SCREEN_MENU_ITEM_LEVEL 1
+#define INTRO_SCREEN_MENU_ITEM_PACK 2
+#define INTRO_SCREEN_MENU_ITEM_HELP 3
+#define INTRO_SCREEN_MENU_ITEM_OPTIONS 4
+#define INTRO_SCREEN_MENU_ITEM_DESIGNER 5
+#define INTRO_SCREEN_MENU_ITEM_EXIT 6
+
 /* demo_mode subfunctions */
 #define DEMO_MODE_INITIALISE 0
 #define DEMO_MODE_TIMEOUT_INITIALISE 1
@@ -105,17 +148,24 @@ int game_cycle_limit;			/* 20, 25, 33 Hz */
 int temp_game_cycle_limit;		/* 20, 25, 33 Hz */
 int cycle_count;				/* A running cycle count used for time stamping objects */
 
-int introscreenmenuposition;
-int helpscreenpage;			
-int helpscreenmenuposition;	
-int optionsscreenpage;		
-int optionsscreenmenuposition[OPTIONS_SCREEN_PAGES];	
+int introscreenselecteditem;
+int helppage;
+int helppageselecteditem;
+int options[104];
+int optionspage;
+int optionspageselecteditem[OPTIONS_SCREEN_PAGES];
 
 /* Function prototypes */
 int my_rand();
 void my_srand(unsigned int seed);
 int demo_mode(int demo_mode_state, int actionid);
-
-
+void manage_game_on_input(int actionid);
+/* Event processors */
+void intro_screen_event_processor(ROB_Event *rob_event);
+void help_screen_event_processor(ROB_Event *rob_event);
+void options_screen_event_processor(ROB_Event *rob_event);
+#ifdef HILDON
+void handleWindowEvent(SDL_Event * event);
+#endif
 
 
