@@ -62,6 +62,9 @@ int level_init(void) {
 	level.colour = UNDEFINED;			/* This is Uint32 so -1 == 0xffffffff and I won't be using that colour */
 	level.colour_override = UNDEFINED;	/* This is Uint32 so -1 == 0xffffffff and I won't be using that colour */
 
+	/* Reset the pointer controls shoot state */
+	pointer_controls.shoot_state = POINTER_CONTROLS_SHOOT_OFF;
+
 	/* Clear board */
 	clear_entire_board();
 
@@ -99,7 +102,7 @@ int level_init(void) {
 		/* Now adjust it to fit within the dimensions of the screen */			
 		if (viewport.w > screen->w / video.field_size) viewport.w = screen->w / video.field_size;
 		/* Set the xoffset and centre the viewport */		
-		viewport.xoffset = (screen->w - viewport.w * video.field_size) / 2;
+		viewport.xoffset = (screen->w - viewport.w * video.field_size) / 2 + video.xshift;
 		
 		/* Set viewport height */
 		if (level.h >= viewport.max_h && !viewport.maximise) {
@@ -110,16 +113,16 @@ int level_init(void) {
 		/* Now adjust it to fit within the dimensions of the screen */			
 		if (viewport.h > screen->h / video.field_size - 3) viewport.h = screen->h / video.field_size - 3;
 		/* Set the yoffset and centre the viewport */		
-		viewport.yoffset = (screen->h - viewport.h * video.field_size) / 2;
+		viewport.yoffset = (screen->h - viewport.h * video.field_size) / 2 + video.yshift;
 
 		/* Set the scoreline screen offset */
-		scoreline.xoffset = (screen->w - video.field_size * DEFAULT_VIEWPORT_WIDTH) / 2;
+		scoreline.xoffset = (screen->w - video.field_size * DEFAULT_VIEWPORT_WIDTH) / 2 + video.xshift;
 		if (scoreline.xoffset < 0) scoreline.xoffset = 0;	/* A special case when screen->w = 240 or 480 */
-		scoreline.yoffset = screen->h / 2 + video.field_size * viewport.h / 2 + video.field_size / 4;
+		scoreline.yoffset = screen->h / 2 + video.field_size * viewport.h / 2 + video.field_size / 4 + video.yshift;
 		
 		/* Set the authorline screen offset */
 		authorline.xoffset = scoreline.xoffset + 2 * video.field_size / 16;
-		authorline.yoffset = screen->h / 2 - video.field_size * viewport.h / 2 - video.field_size - video.field_size / 4;
+		authorline.yoffset = screen->h / 2 - video.field_size * viewport.h / 2 - video.field_size - video.field_size / 4 + video.yshift;
 		
 		/* Start scrolling from the opposite end of the board to Robbo */
 		if (robbo.x < level.w / 2) {
@@ -454,7 +457,8 @@ int load_level_data(int level_number) {
 						break;
 					} else {
 						board[x][y].type = transform_char(line[x]);
-						if (board[x][y].type != EMPTY_FIELD) create_object(x, y, board[x][y].type);
+						create_object(x, y, board[x][y].type);
+						/*if (board[x][y].type != EMPTY_FIELD) create_object(x, y, board[x][y].type); Redundant */
 						switch (board[x][y].type) {
 							case SCREW:
 								robbo.screws++;
