@@ -241,7 +241,7 @@ void load_selected_locale(void) {
 	FILE *fp;
 	char filename[256], line[1024];	/* 1024 because if fgets returns a partial line then line_count will be incremented again */
 	int count, count2, line_count = 0, tag_found, tag_index, row = 0;
-	#define CONFIG_ITEM_COUNT 105
+	#define CONFIG_ITEM_COUNT 126
 	struct config_item {
 		char name[60];
 		int state;
@@ -270,7 +270,6 @@ void load_selected_locale(void) {
 	strcpy(config_items[++row].name, "[txt_intro_menu]"); config_items[row].destination = txt_intro_menu; config_items[row].max_rows = TXT_INTRO_MENU_ROWS; 
 	strcpy(config_items[++row].name, "[txt_of]"); config_items[row].destination = txt_of; 
 	strcpy(config_items[++row].name, "[txt_intro_credits]"); config_items[row].destination = txt_intro_credits; config_items[row].max_rows = TXT_INTRO_CREDITS_ROWS; 
-
 	strcpy(config_items[++row].name, "[txt_help_page0]"); config_items[row].destination = txt_help_page0; config_items[row].max_rows = TXT_HELP_PAGE0_ROWS; 
 	strcpy(config_items[++row].name, "[txt_help_page1]"); config_items[row].destination = txt_help_page1; config_items[row].max_rows = TXT_HELP_PAGE1_ROWS; 
 	strcpy(config_items[++row].name, "[txt_help_page2]"); config_items[row].destination = txt_help_page2; config_items[row].max_rows = TXT_HELP_PAGE2_ROWS; 
@@ -278,7 +277,6 @@ void load_selected_locale(void) {
 	strcpy(config_items[++row].name, "[txt_Back]"); config_items[row].destination = txt_Back; 
 	strcpy(config_items[++row].name, "[txt_Next]"); config_items[row].destination = txt_Next; 
 	strcpy(config_items[++row].name, "[txt_Exit]"); config_items[row].destination = txt_Exit; 
-
 	strcpy(config_items[++row].name, "[txt_options_page1]"); config_items[row].destination = txt_options_page1; config_items[row].max_rows = TXT_OPTIONS_PAGE1_ROWS; 
 	strcpy(config_items[++row].name, "[txt_Save]"); config_items[row].destination = txt_Save; 
 	strcpy(config_items[++row].name, "[txt_Game_Speed]"); config_items[row].destination = txt_Game_Speed; 
@@ -293,10 +291,19 @@ void load_selected_locale(void) {
 	strcpy(config_items[++row].name, "[txt_Joystick_Axes_Dead_Zone]"); config_items[row].destination = txt_Joystick_Axes_Dead_Zone; 
 	strcpy(config_items[++row].name, "[txt_Language]"); config_items[row].destination = txt_Language; 
 	strcpy(config_items[++row].name, "[txt_Translation_by]"); config_items[row].destination = txt_Translation_by; 
+	strcpy(config_items[++row].name, "[txt_Save_Frequency]"); config_items[row].destination = txt_Save_Frequency; 
+	strcpy(config_items[++row].name, "[txt_On_Exit]"); config_items[row].destination = txt_On_Exit; 
+	strcpy(config_items[++row].name, "[txt_On_Change]"); config_items[row].destination = txt_On_Change; 
+
 	strcpy(config_items[++row].name, "[txt_Move_Up]"); config_items[row].destination = txt_Move_Up; 
-	strcpy(config_items[++row].name, "[txt_Move_Down]"); config_items[row].destination = txt_Move_Down; 
-	strcpy(config_items[++row].name, "[txt_Move_Left]"); config_items[row].destination = txt_Move_Left; 
+	strcpy(config_items[++row].name, "[txt_Move_Up_Right]"); config_items[row].destination = txt_Move_Up_Right; 
 	strcpy(config_items[++row].name, "[txt_Move_Right]"); config_items[row].destination = txt_Move_Right; 
+	strcpy(config_items[++row].name, "[txt_Move_Down_Right]"); config_items[row].destination = txt_Move_Down_Right; 
+	strcpy(config_items[++row].name, "[txt_Move_Down]"); config_items[row].destination = txt_Move_Down; 
+	strcpy(config_items[++row].name, "[txt_Move_Down_Left]"); config_items[row].destination = txt_Move_Down_Left; 
+	strcpy(config_items[++row].name, "[txt_Move_Left]"); config_items[row].destination = txt_Move_Left; 
+	strcpy(config_items[++row].name, "[txt_Move_Up_Left]"); config_items[row].destination = txt_Move_Up_Left; 
+
 	strcpy(config_items[++row].name, "[txt_Shoot_Up]"); config_items[row].destination = txt_Shoot_Up; 
 	strcpy(config_items[++row].name, "[txt_Shoot_Down]"); config_items[row].destination = txt_Shoot_Down; 
 	strcpy(config_items[++row].name, "[txt_Shoot_Left]"); config_items[row].destination = txt_Shoot_Left; 
@@ -318,21 +325,32 @@ void load_selected_locale(void) {
 	strcpy(config_items[++row].name, "[txt_Volume_Down]"); config_items[row].destination = txt_Volume_Down; 
 	strcpy(config_items[++row].name, "[txt_Modifier]"); config_items[row].destination = txt_Modifier; 
 	strcpy(config_items[++row].name, "[txt_Restore_Default_Controls]"); config_items[row].destination = txt_Restore_Default_Controls; 
+	strcpy(config_items[++row].name, "[txt_Toggle_Designer]"); config_items[row].destination = txt_Toggle_Designer; 
+	strcpy(config_items[++row].name, "[txt_Scroll_Up]"); config_items[row].destination = txt_Scroll_Up; 
+	strcpy(config_items[++row].name, "[txt_Scroll_Down]"); config_items[row].destination = txt_Scroll_Down; 
+	strcpy(config_items[++row].name, "[txt_Primary_Click]"); config_items[row].destination = txt_Primary_Click;
+	strcpy(config_items[++row].name, "[txt_System_Pointer"); config_items[row].destination = txt_System_Pointer;
+	strcpy(config_items[++row].name, "[txt_Disabled]"); config_items[row].destination = txt_Disabled;
+	strcpy(config_items[++row].name, "[txt_Enabled]"); config_items[row].destination = txt_Enabled;
+	strcpy(config_items[++row].name, "[txt_Simulated_Pointer]"); config_items[row].destination = txt_Simulated_Pointer;
 	strcpy(config_items[++row].name, "[txt_Key]"); config_items[row].destination = txt_Key; 
 	strcpy(config_items[++row].name, "[txt_Joy]"); config_items[row].destination = txt_Joy; 
+	strcpy(config_items[++row].name, "[txt_Ptr]"); config_items[row].destination = txt_Ptr; 
 	strcpy(config_items[++row].name, "[txt_Mod]"); config_items[row].destination = txt_Mod; 
 	strcpy(config_items[++row].name, "[txt_Skin]"); config_items[row].destination = txt_Skin; 
 	strcpy(config_items[++row].name, "[txt_By]"); config_items[row].destination = txt_By; 
-
 	strcpy(config_items[++row].name, "[txt_Level_Author]"); config_items[row].destination = txt_Level_Author; 
 	strcpy(config_items[++row].name, "[txt_endscreen_congrats]"); config_items[row].destination = txt_endscreen_congrats; config_items[row].max_rows = TXT_ENDSCREEN_CONGRATS_ROWS; 
-
 	strcpy(config_items[++row].name, "[txt_Volume]"); config_items[row].destination = txt_Volume; 
 	strcpy(config_items[++row].name, "[txt_Changes_saved]"); config_items[row].destination = txt_Changes_saved; 
 	strcpy(config_items[++row].name, "[txt_Default_controls_restored]"); config_items[row].destination = txt_Default_controls_restored; 
 	strcpy(config_items[++row].name, "[txt_Press_something_and_release]"); config_items[row].destination = txt_Press_something_and_release; 
 	strcpy(config_items[++row].name, "[txt_No_input_was_detected]"); config_items[row].destination = txt_No_input_was_detected; 
-
+	strcpy(config_items[++row].name, "[txt_ptr_BtnLeft]"); config_items[row].destination = txt_ptr_BtnLeft; 
+	strcpy(config_items[++row].name, "[txt_ptr_BtnMiddle]"); config_items[row].destination = txt_ptr_BtnMiddle; 
+	strcpy(config_items[++row].name, "[txt_ptr_BtnRight]"); config_items[row].destination = txt_ptr_BtnRight; 
+	strcpy(config_items[++row].name, "[txt_ptr_WheelUp]"); config_items[row].destination = txt_ptr_WheelUp; 
+	strcpy(config_items[++row].name, "[txt_ptr_WheelDown]"); config_items[row].destination = txt_ptr_WheelDown; 
 	strcpy(config_items[++row].name, "[txt_key_Backspace]"); config_items[row].destination = txt_key_Backspace; 
 	strcpy(config_items[++row].name, "[txt_key_Tab]"); config_items[row].destination = txt_key_Tab; 
 	strcpy(config_items[++row].name, "[txt_key_Clear]"); config_items[row].destination = txt_key_Clear; 
