@@ -29,6 +29,7 @@
 /*
 #define DEBUG_READ_RESOURCE_FILE
 */
+
 #define UNDEFINED2 -2
 
 /* Variables */
@@ -55,10 +56,13 @@ int read_resource_file(char *filename) {
 	int read_key_repeat_delay = UNDEFINED, read_key_repeat_interval = UNDEFINED;
 	char read_selected_skin[100], read_selected_locale[100];
 	int read_sensible_bears = UNDEFINED, read_sensible_questionmarks = UNDEFINED, read_sensible_solid_lasers = UNDEFINED;
+	int read_save_frequency = UNDEFINED;
 	struct pack read_level_packs[MAX_LEVEL_PACKS];
 	int read_pack_count = 0, count, count2, error_found = FALSE, found;
 	struct control read_user_controls[USER_CONTROLS];
+	ROB_OpEnv read_op_env;
 	char error_setting[50];
+	struct pointercontrols read_pointer_controls;
 	
 	strcpy(read_default_joystick_name, "-1");
 	strcpy(read_selected_skin, "-1");
@@ -85,6 +89,19 @@ int read_resource_file(char *filename) {
 		read_user_controls[count].mod = UNDEFINED2;
 	}
 
+	/* Reset read_op_env so we can test for missing settings */
+	read_op_env.systempointer = UNDEFINED;
+	read_op_env.pointer = UNDEFINED;
+	read_op_env.pointer_mode = UNDEFINED;
+	read_op_env.pointer_move_unit_low = UNDEFINED;
+	read_op_env.pointer_move_unit_high = UNDEFINED;
+	read_op_env.pointer_move_unit_threshold = UNDEFINED;
+
+	/* Reset read_pointer_controls so we can test for missing settings */
+	read_pointer_controls.state = UNDEFINED;
+	read_pointer_controls.shoot_state = UNDEFINED;
+	read_pointer_controls.pad_type = UNDEFINED;
+
 	/* Read the rc data into read_level_packs */
 	while (fscanf(fp, "%s", line) == 1) {
 		if (!strncmp(line, "[game_cycle_limit]", 18)) {
@@ -111,6 +128,14 @@ int read_resource_file(char *filename) {
 			fscanf(fp, "%s", read_selected_skin);
 		} else if (!strncmp(line, "[selected_locale]", 17)) {
 			fscanf(fp, "%s", read_selected_locale);
+		} else if (!strncmp(line, "[save_type]", 11) || !strncmp(line, "[save_frequency]", 16)) {	/* save_type was changed to save_frequency in v0.63 and this fix can eventually be removed. temp temp */
+			fscanf(fp, "%i", &read_save_frequency);
+		} else if (!strncmp(line, "[op_env.systempointer]", 22)) {
+			fscanf(fp, "%i", &read_op_env.systempointer);
+		} else if (!strncmp(line, "[pointer_controls.state]", 24)) {
+			fscanf(fp, "%i", &read_pointer_controls.state);
+		} else if (!strncmp(line, "[pointer_controls.pad_type]", 27)) {
+			fscanf(fp, "%i", &read_pointer_controls.pad_type);
 		} else if (!strncmp(line, "[sensible_bears]", 16)) {
 			fscanf(fp, "%i", &read_sensible_bears);
 		} else if (!strncmp(line, "[sensible_questionmarks]", 24)) {
@@ -133,24 +158,54 @@ int read_resource_file(char *filename) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_UP].id);
 		} else if (!strncmp(line, "[ACTION_UP.mod]", 15)) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_UP].mod);
-		} else if (!strncmp(line, "[ACTION_DOWN.device]", 20)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_DOWN].device);
-		} else if (!strncmp(line, "[ACTION_DOWN.id]", 16)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_DOWN].id);
-		} else if (!strncmp(line, "[ACTION_DOWN.mod]", 17)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_DOWN].mod);
-		} else if (!strncmp(line, "[ACTION_LEFT.device]", 20)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_LEFT].device);
-		} else if (!strncmp(line, "[ACTION_LEFT.id]", 16)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_LEFT].id);
-		} else if (!strncmp(line, "[ACTION_LEFT.mod]", 17)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_LEFT].mod);
+		} else if (!strncmp(line, "[ACTION_UP_RIGHT.device]", 24)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_UP_RIGHT].device);
+		} else if (!strncmp(line, "[ACTION_UP_RIGHT.id]", 20)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_UP_RIGHT].id);
+		} else if (!strncmp(line, "[ACTION_UP_RIGHT.mod]", 21)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_UP_RIGHT].mod);
 		} else if (!strncmp(line, "[ACTION_RIGHT.device]", 21)) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_RIGHT].device);
 		} else if (!strncmp(line, "[ACTION_RIGHT.id]", 17)) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_RIGHT].id);
 		} else if (!strncmp(line, "[ACTION_RIGHT.mod]", 18)) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_RIGHT].mod);
+		} else if (!strncmp(line, "[ACTION_DOWN_RIGHT.device]", 26)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_DOWN_RIGHT].device);
+		} else if (!strncmp(line, "[ACTION_DOWN_RIGHT.id]", 22)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_DOWN_RIGHT].id);
+		} else if (!strncmp(line, "[ACTION_DOWN_RIGHT.mod]", 23)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_DOWN_RIGHT].mod);
+		} else if (!strncmp(line, "[ACTION_DOWN.device]", 20)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_DOWN].device);
+		} else if (!strncmp(line, "[ACTION_DOWN.id]", 16)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_DOWN].id);
+		} else if (!strncmp(line, "[ACTION_DOWN.mod]", 17)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_DOWN].mod);
+		} else if (!strncmp(line, "[ACTION_DOWN_LEFT.device]", 25)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_DOWN_LEFT].device);
+		} else if (!strncmp(line, "[ACTION_DOWN_LEFT.id]", 21)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_DOWN_LEFT].id);
+		} else if (!strncmp(line, "[ACTION_DOWN_LEFT.mod]", 22)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_DOWN_LEFT].mod);
+		} else if (!strncmp(line, "[ACTION_LEFT.device]", 20)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_LEFT].device);
+		} else if (!strncmp(line, "[ACTION_LEFT.id]", 16)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_LEFT].id);
+		} else if (!strncmp(line, "[ACTION_LEFT.mod]", 17)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_LEFT].mod);
+		} else if (!strncmp(line, "[ACTION_UP_LEFT.device]", 23)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_UP_LEFT].device);
+		} else if (!strncmp(line, "[ACTION_UP_LEFT.id]", 19)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_UP_LEFT].id);
+		} else if (!strncmp(line, "[ACTION_UP_LEFT.mod]", 20)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_UP_LEFT].mod);
+		} else if (!strncmp(line, "[ACTION_RESTART.device]", 23)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_RESTART].device);
+		} else if (!strncmp(line, "[ACTION_RESTART.id]", 19)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_RESTART].id);
+		} else if (!strncmp(line, "[ACTION_RESTART.mod]", 20)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_RESTART].mod);
 		} else if (!strncmp(line, "[ACTION_SHOOT_UP.device]", 24)) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_SHOOT_UP].device);
 		} else if (!strncmp(line, "[ACTION_SHOOT_UP.id]", 20)) {
@@ -199,12 +254,12 @@ int read_resource_file(char *filename) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_OPTIONS].id);
 		} else if (!strncmp(line, "[ACTION_OPTIONS.mod]", 20)) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_OPTIONS].mod);
-		} else if (!strncmp(line, "[ACTION_RESTART.device]", 23)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_RESTART].device);
-		} else if (!strncmp(line, "[ACTION_RESTART.id]", 19)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_RESTART].id);
-		} else if (!strncmp(line, "[ACTION_RESTART.mod]", 20)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_RESTART].mod);
+		} else if (!strncmp(line, "[ACTION_TOGGLE_FULLSCREEN.device]", 33)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_TOGGLE_FULLSCREEN].device);
+		} else if (!strncmp(line, "[ACTION_TOGGLE_FULLSCREEN.id]", 29)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_TOGGLE_FULLSCREEN].id);
+		} else if (!strncmp(line, "[ACTION_TOGGLE_FULLSCREEN.mod]", 30)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_TOGGLE_FULLSCREEN].mod);
 		} else if (!strncmp(line, "[ACTION_PREVIOUS_LEVEL.device]", 30)) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_PREVIOUS_LEVEL].device);
 		} else if (!strncmp(line, "[ACTION_PREVIOUS_LEVEL.id]", 26)) {
@@ -229,24 +284,6 @@ int read_resource_file(char *filename) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_NEXT_PACK].id);
 		} else if (!strncmp(line, "[ACTION_NEXT_PACK.mod]", 22)) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_NEXT_PACK].mod);
-		} else if (!strncmp(line, "[ACTION_MODIFIER1.device]", 25)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER1].device);
-		} else if (!strncmp(line, "[ACTION_MODIFIER1.id]", 21)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER1].id);
-		} else if (!strncmp(line, "[ACTION_MODIFIER1.mod]", 22)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER1].mod);
-		} else if (!strncmp(line, "[ACTION_MODIFIER2.device]", 25)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER2].device);
-		} else if (!strncmp(line, "[ACTION_MODIFIER2.id]", 21)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER2].id);
-		} else if (!strncmp(line, "[ACTION_MODIFIER2.mod]", 22)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER2].mod);
-		} else if (!strncmp(line, "[ACTION_TOGGLE_FULLSCREEN.device]", 33)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_TOGGLE_FULLSCREEN].device);
-		} else if (!strncmp(line, "[ACTION_TOGGLE_FULLSCREEN.id]", 29)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_TOGGLE_FULLSCREEN].id);
-		} else if (!strncmp(line, "[ACTION_TOGGLE_FULLSCREEN.mod]", 30)) {
-			fscanf(fp, "%i", &read_user_controls[ACTION_TOGGLE_FULLSCREEN].mod);
 		} else if (!strncmp(line, "[ACTION_HOME.device]", 20)) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_HOME].device);
 		} else if (!strncmp(line, "[ACTION_HOME.id]", 16)) {
@@ -271,6 +308,12 @@ int read_resource_file(char *filename) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_PAGEDOWN].id);
 		} else if (!strncmp(line, "[ACTION_PAGEDOWN.mod]", 21)) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_PAGEDOWN].mod);
+		} else if (!strncmp(line, "[ACTION_TOGGLE_DESIGNER.device]", 31)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_TOGGLE_DESIGNER].device);
+		} else if (!strncmp(line, "[ACTION_TOGGLE_DESIGNER.id]", 27)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_TOGGLE_DESIGNER].id);
+		} else if (!strncmp(line, "[ACTION_TOGGLE_DESIGNER.mod]", 28)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_TOGGLE_DESIGNER].mod);
 		} else if (!strncmp(line, "[ACTION_VOLUP.device]", 21)) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_VOLUP].device);
 		} else if (!strncmp(line, "[ACTION_VOLUP.id]", 17)) {
@@ -283,11 +326,59 @@ int read_resource_file(char *filename) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_VOLDOWN].id);
 		} else if (!strncmp(line, "[ACTION_VOLDOWN.mod]", 20)) {
 			fscanf(fp, "%i", &read_user_controls[ACTION_VOLDOWN].mod);
+		} else if (!strncmp(line, "[ACTION_MODIFIER1.device]", 25)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER1].device);
+		} else if (!strncmp(line, "[ACTION_MODIFIER1.id]", 21)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER1].id);
+		} else if (!strncmp(line, "[ACTION_MODIFIER1.mod]", 22)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER1].mod);
+		} else if (!strncmp(line, "[ACTION_MODIFIER2.device]", 25)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER2].device);
+		} else if (!strncmp(line, "[ACTION_MODIFIER2.id]", 21)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER2].id);
+		} else if (!strncmp(line, "[ACTION_MODIFIER2.mod]", 22)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER2].mod);
+		} else if (!strncmp(line, "[ACTION_MODIFIER3.device]", 25)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER3].device);
+		} else if (!strncmp(line, "[ACTION_MODIFIER3.id]", 21)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER3].id);
+		} else if (!strncmp(line, "[ACTION_MODIFIER3.mod]", 22)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER3].mod);
+		} else if (!strncmp(line, "[ACTION_MODIFIER4.device]", 25)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER4].device);
+		} else if (!strncmp(line, "[ACTION_MODIFIER4.id]", 21)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER4].id);
+		} else if (!strncmp(line, "[ACTION_MODIFIER4.mod]", 22)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_MODIFIER4].mod);
+		} else if (!strncmp(line, "[ACTION_SCROLL_UP.device]", 25)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_SCROLL_UP].device);
+		} else if (!strncmp(line, "[ACTION_SCROLL_UP.id]", 21)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_SCROLL_UP].id);
+		} else if (!strncmp(line, "[ACTION_SCROLL_UP.mod]", 22)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_SCROLL_UP].mod);
+		} else if (!strncmp(line, "[ACTION_SCROLL_DOWN.device]", 27)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_SCROLL_DOWN].device);
+		} else if (!strncmp(line, "[ACTION_SCROLL_DOWN.id]", 23)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_SCROLL_DOWN].id);
+		} else if (!strncmp(line, "[ACTION_SCROLL_DOWN.mod]", 24)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_SCROLL_DOWN].mod);
+		} else if (!strncmp(line, "[ACTION_PRIMARY_CLICK.device]", 29)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_PRIMARY_CLICK].device);
+		} else if (!strncmp(line, "[ACTION_PRIMARY_CLICK.id]", 25)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_PRIMARY_CLICK].id);
+		} else if (!strncmp(line, "[ACTION_PRIMARY_CLICK.mod]", 26)) {
+			fscanf(fp, "%i", &read_user_controls[ACTION_PRIMARY_CLICK].mod);
 		}
 	}
 
 	fclose(fp);
 
+	/* ACTION_MODIFIER1 & ACTION_MODIFIER2 were moved in v0.63 and this fix can eventually be removed. temp temp */
+	for (count = 0; count < USER_CONTROLS; count++) {
+		if (read_user_controls[count].mod == 24) read_user_controls[count].mod = ACTION_MODIFIER1;
+		if (read_user_controls[count].mod == 25) read_user_controls[count].mod = ACTION_MODIFIER2;
+	}
+
 	#ifdef DEBUG_READ_RESOURCE_FILE
 		printf("*** Start %s ***\n", __func__);
 		printf("read_game_cycle_limit=%i\n", read_game_cycle_limit);
@@ -299,6 +390,10 @@ int read_resource_file(char *filename) {
 		printf("read_key_repeat_interval=%i\n", read_key_repeat_interval);
 		printf("read_selected_skin=%s\n", read_selected_skin);
 		printf("read_selected_locale=%s\n", read_selected_locale);
+		printf("read_save_frequency=%i\n", read_save_frequency);
+		printf("read_op_env.systempointer=%i\n", read_op_env.systempointer);
+		printf("read_pointer_controls.state=%i\n", read_pointer_controls.state);
+		printf("read_pointer_controls.pad_type=%i\n", read_pointer_controls.pad_type);
 		printf("read_sensible_bears=%i\n", read_sensible_bears);
 		printf("read_sensible_questionmarks=%i\n", read_sensible_questionmarks);
 		printf("read_sensible_solid_lasers=%i\n", read_sensible_solid_lasers);
@@ -373,6 +468,13 @@ int read_resource_file(char *filename) {
 			}
 		}
 	}
+	/* rcfile save frequency */
+	if (read_save_frequency != UNDEFINED) rcfile.save_frequency = read_save_frequency;
+	/* System pointer state */
+	if (read_op_env.systempointer != UNDEFINED) gnurobbo_op_env.systempointer = read_op_env.systempointer;
+	/* Pointer controls */
+	if (read_pointer_controls.state != UNDEFINED) pointer_controls.state = read_pointer_controls.state;
+	if (read_pointer_controls.pad_type != UNDEFINED) pointer_controls.pad_type = read_pointer_controls.pad_type;
 	/* Game mechanics */
 	if (read_sensible_bears != UNDEFINED) game_mechanics.sensible_bears = read_sensible_bears;
 	if (read_sensible_questionmarks != UNDEFINED) game_mechanics.sensible_questionmarks = read_sensible_questionmarks;
@@ -451,9 +553,10 @@ int read_resource_file(char *filename) {
 /* Saves resource file data.
 
    On entry: filename is the full path to the rc file
+			 inform = true to inform the user via the console
 	On exit: returns 1 on error */   
 
-int save_resource_file(char *filename) {
+int save_resource_file(char *filename, int inform) {
 	FILE *fp;
 	int count;
 
@@ -462,7 +565,9 @@ int save_resource_file(char *filename) {
 		return 1;
 	}
 
-	fprintf(stdout, "Writing to %s\n", filename);
+	if (inform) fprintf(stdout, "Writing to %s\n", filename);
+
+	fprintf(fp, "[version]\n%s\n", VERSION);
 
 	fprintf(fp, "[game_cycle_limit]\n%i\n", game_cycle_limit);
 	fprintf(fp, "[game_cycle_delay]\n%i\n", game_cycle_delay);
@@ -477,11 +582,12 @@ int save_resource_file(char *filename) {
 
 	fprintf(fp, "[key_repeat_delay]\n%i\n", key_repeat_delay);
 	fprintf(fp, "[key_repeat_interval]\n%i\n", key_repeat_interval);
-
 	fprintf(fp, "[selected_skin]\n%s\n", skins[selected_skin].foldername);
-
 	fprintf(fp, "[selected_locale]\n%s\n", locales[selected_locale].foldername);
-
+	fprintf(fp, "[save_frequency]\n%i\n", rcfile.save_frequency);
+	fprintf(fp, "[op_env.systempointer]\n%i\n", gnurobbo_op_env.systempointer);
+	fprintf(fp, "[pointer_controls.state]\n%i\n", pointer_controls.state);
+	fprintf(fp, "[pointer_controls.pad_type]\n%i\n", pointer_controls.pad_type);
 	fprintf(fp, "[sensible_bears]\n%i\n", game_mechanics.sensible_bears);
 	fprintf(fp, "[sensible_questionmarks]\n%i\n", game_mechanics.sensible_questionmarks);
 	fprintf(fp, "[sensible_solid_lasers]\n%i\n", game_mechanics.sensible_solid_lasers);
@@ -496,15 +602,31 @@ int save_resource_file(char *filename) {
 	fprintf(fp, "[ACTION_UP.device]\n%i\n", user_controls[ACTION_UP].device);
 	fprintf(fp, "[ACTION_UP.id]\n%i\n", user_controls[ACTION_UP].id);
 	fprintf(fp, "[ACTION_UP.mod]\n%i\n", user_controls[ACTION_UP].mod);
-	fprintf(fp, "[ACTION_DOWN.device]\n%i\n", user_controls[ACTION_DOWN].device);
-	fprintf(fp, "[ACTION_DOWN.id]\n%i\n", user_controls[ACTION_DOWN].id);
-	fprintf(fp, "[ACTION_DOWN.mod]\n%i\n", user_controls[ACTION_DOWN].mod);
-	fprintf(fp, "[ACTION_LEFT.device]\n%i\n", user_controls[ACTION_LEFT].device);
-	fprintf(fp, "[ACTION_LEFT.id]\n%i\n", user_controls[ACTION_LEFT].id);
-	fprintf(fp, "[ACTION_LEFT.mod]\n%i\n", user_controls[ACTION_LEFT].mod);
+	fprintf(fp, "[ACTION_UP_RIGHT.device]\n%i\n", user_controls[ACTION_UP_RIGHT].device);
+	fprintf(fp, "[ACTION_UP_RIGHT.id]\n%i\n", user_controls[ACTION_UP_RIGHT].id);
+	fprintf(fp, "[ACTION_UP_RIGHT.mod]\n%i\n", user_controls[ACTION_UP_RIGHT].mod);
 	fprintf(fp, "[ACTION_RIGHT.device]\n%i\n", user_controls[ACTION_RIGHT].device);
 	fprintf(fp, "[ACTION_RIGHT.id]\n%i\n", user_controls[ACTION_RIGHT].id);
 	fprintf(fp, "[ACTION_RIGHT.mod]\n%i\n", user_controls[ACTION_RIGHT].mod);
+	fprintf(fp, "[ACTION_DOWN_RIGHT.device]\n%i\n", user_controls[ACTION_DOWN_RIGHT].device);
+	fprintf(fp, "[ACTION_DOWN_RIGHT.id]\n%i\n", user_controls[ACTION_DOWN_RIGHT].id);
+	fprintf(fp, "[ACTION_DOWN_RIGHT.mod]\n%i\n", user_controls[ACTION_DOWN_RIGHT].mod);
+	fprintf(fp, "[ACTION_DOWN.device]\n%i\n", user_controls[ACTION_DOWN].device);
+	fprintf(fp, "[ACTION_DOWN.id]\n%i\n", user_controls[ACTION_DOWN].id);
+	fprintf(fp, "[ACTION_DOWN.mod]\n%i\n", user_controls[ACTION_DOWN].mod);
+	fprintf(fp, "[ACTION_DOWN_LEFT.device]\n%i\n", user_controls[ACTION_DOWN_LEFT].device);
+	fprintf(fp, "[ACTION_DOWN_LEFT.id]\n%i\n", user_controls[ACTION_DOWN_LEFT].id);
+	fprintf(fp, "[ACTION_DOWN_LEFT.mod]\n%i\n", user_controls[ACTION_DOWN_LEFT].mod);
+	fprintf(fp, "[ACTION_LEFT.device]\n%i\n", user_controls[ACTION_LEFT].device);
+	fprintf(fp, "[ACTION_LEFT.id]\n%i\n", user_controls[ACTION_LEFT].id);
+	fprintf(fp, "[ACTION_LEFT.mod]\n%i\n", user_controls[ACTION_LEFT].mod);
+	fprintf(fp, "[ACTION_UP_LEFT.device]\n%i\n", user_controls[ACTION_UP_LEFT].device);
+	fprintf(fp, "[ACTION_UP_LEFT.id]\n%i\n", user_controls[ACTION_UP_LEFT].id);
+	fprintf(fp, "[ACTION_UP_LEFT.mod]\n%i\n", user_controls[ACTION_UP_LEFT].mod);
+	fprintf(fp, "[ACTION_RESTART.device]\n%i\n", user_controls[ACTION_RESTART].device);
+	fprintf(fp, "[ACTION_RESTART.id]\n%i\n", user_controls[ACTION_RESTART].id);
+	fprintf(fp, "[ACTION_RESTART.mod]\n%i\n", user_controls[ACTION_RESTART].mod);
+
 	fprintf(fp, "[ACTION_SHOOT_UP.device]\n%i\n", user_controls[ACTION_SHOOT_UP].device);
 	fprintf(fp, "[ACTION_SHOOT_UP.id]\n%i\n", user_controls[ACTION_SHOOT_UP].id);
 	fprintf(fp, "[ACTION_SHOOT_UP.mod]\n%i\n", user_controls[ACTION_SHOOT_UP].mod);
@@ -529,9 +651,10 @@ int save_resource_file(char *filename) {
 	fprintf(fp, "[ACTION_OPTIONS.device]\n%i\n", user_controls[ACTION_OPTIONS].device);
 	fprintf(fp, "[ACTION_OPTIONS.id]\n%i\n", user_controls[ACTION_OPTIONS].id);
 	fprintf(fp, "[ACTION_OPTIONS.mod]\n%i\n", user_controls[ACTION_OPTIONS].mod);
-	fprintf(fp, "[ACTION_RESTART.device]\n%i\n", user_controls[ACTION_RESTART].device);
-	fprintf(fp, "[ACTION_RESTART.id]\n%i\n", user_controls[ACTION_RESTART].id);
-	fprintf(fp, "[ACTION_RESTART.mod]\n%i\n", user_controls[ACTION_RESTART].mod);
+	fprintf(fp, "[ACTION_TOGGLE_FULLSCREEN.device]\n%i\n", user_controls[ACTION_TOGGLE_FULLSCREEN].device);
+	fprintf(fp, "[ACTION_TOGGLE_FULLSCREEN.id]\n%i\n", user_controls[ACTION_TOGGLE_FULLSCREEN].id);
+	fprintf(fp, "[ACTION_TOGGLE_FULLSCREEN.mod]\n%i\n", user_controls[ACTION_TOGGLE_FULLSCREEN].mod);
+
 	fprintf(fp, "[ACTION_PREVIOUS_LEVEL.device]\n%i\n", user_controls[ACTION_PREVIOUS_LEVEL].device);
 	fprintf(fp, "[ACTION_PREVIOUS_LEVEL.id]\n%i\n", user_controls[ACTION_PREVIOUS_LEVEL].id);
 	fprintf(fp, "[ACTION_PREVIOUS_LEVEL.mod]\n%i\n", user_controls[ACTION_PREVIOUS_LEVEL].mod);
@@ -544,15 +667,6 @@ int save_resource_file(char *filename) {
 	fprintf(fp, "[ACTION_NEXT_PACK.device]\n%i\n", user_controls[ACTION_NEXT_PACK].device);
 	fprintf(fp, "[ACTION_NEXT_PACK.id]\n%i\n", user_controls[ACTION_NEXT_PACK].id);
 	fprintf(fp, "[ACTION_NEXT_PACK.mod]\n%i\n", user_controls[ACTION_NEXT_PACK].mod);
-	fprintf(fp, "[ACTION_MODIFIER1.device]\n%i\n", user_controls[ACTION_MODIFIER1].device);
-	fprintf(fp, "[ACTION_MODIFIER1.id]\n%i\n", user_controls[ACTION_MODIFIER1].id);
-	fprintf(fp, "[ACTION_MODIFIER1.mod]\n%i\n", user_controls[ACTION_MODIFIER1].mod);
-	fprintf(fp, "[ACTION_MODIFIER2.device]\n%i\n", user_controls[ACTION_MODIFIER2].device);
-	fprintf(fp, "[ACTION_MODIFIER2.id]\n%i\n", user_controls[ACTION_MODIFIER2].id);
-	fprintf(fp, "[ACTION_MODIFIER2.mod]\n%i\n", user_controls[ACTION_MODIFIER2].mod);
-	fprintf(fp, "[ACTION_TOGGLE_FULLSCREEN.device]\n%i\n", user_controls[ACTION_TOGGLE_FULLSCREEN].device);
-	fprintf(fp, "[ACTION_TOGGLE_FULLSCREEN.id]\n%i\n", user_controls[ACTION_TOGGLE_FULLSCREEN].id);
-	fprintf(fp, "[ACTION_TOGGLE_FULLSCREEN.mod]\n%i\n", user_controls[ACTION_TOGGLE_FULLSCREEN].mod);
 	fprintf(fp, "[ACTION_HOME.device]\n%i\n", user_controls[ACTION_HOME].device);
 	fprintf(fp, "[ACTION_HOME.id]\n%i\n", user_controls[ACTION_HOME].id);
 	fprintf(fp, "[ACTION_HOME.mod]\n%i\n", user_controls[ACTION_HOME].mod);
@@ -565,12 +679,37 @@ int save_resource_file(char *filename) {
 	fprintf(fp, "[ACTION_PAGEDOWN.device]\n%i\n", user_controls[ACTION_PAGEDOWN].device);
 	fprintf(fp, "[ACTION_PAGEDOWN.id]\n%i\n", user_controls[ACTION_PAGEDOWN].id);
 	fprintf(fp, "[ACTION_PAGEDOWN.mod]\n%i\n", user_controls[ACTION_PAGEDOWN].mod);
+	fprintf(fp, "[ACTION_TOGGLE_DESIGNER.device]\n%i\n", user_controls[ACTION_TOGGLE_DESIGNER].device);
+	fprintf(fp, "[ACTION_TOGGLE_DESIGNER.id]\n%i\n", user_controls[ACTION_TOGGLE_DESIGNER].id);
+	fprintf(fp, "[ACTION_TOGGLE_DESIGNER.mod]\n%i\n", user_controls[ACTION_TOGGLE_DESIGNER].mod);
+
 	fprintf(fp, "[ACTION_VOLUP.device]\n%i\n", user_controls[ACTION_VOLUP].device);
 	fprintf(fp, "[ACTION_VOLUP.id]\n%i\n", user_controls[ACTION_VOLUP].id);
 	fprintf(fp, "[ACTION_VOLUP.mod]\n%i\n", user_controls[ACTION_VOLUP].mod);
 	fprintf(fp, "[ACTION_VOLDOWN.device]\n%i\n", user_controls[ACTION_VOLDOWN].device);
 	fprintf(fp, "[ACTION_VOLDOWN.id]\n%i\n", user_controls[ACTION_VOLDOWN].id);
 	fprintf(fp, "[ACTION_VOLDOWN.mod]\n%i\n", user_controls[ACTION_VOLDOWN].mod);
+	fprintf(fp, "[ACTION_MODIFIER1.device]\n%i\n", user_controls[ACTION_MODIFIER1].device);
+	fprintf(fp, "[ACTION_MODIFIER1.id]\n%i\n", user_controls[ACTION_MODIFIER1].id);
+	fprintf(fp, "[ACTION_MODIFIER1.mod]\n%i\n", user_controls[ACTION_MODIFIER1].mod);
+	fprintf(fp, "[ACTION_MODIFIER2.device]\n%i\n", user_controls[ACTION_MODIFIER2].device);
+	fprintf(fp, "[ACTION_MODIFIER2.id]\n%i\n", user_controls[ACTION_MODIFIER2].id);
+	fprintf(fp, "[ACTION_MODIFIER2.mod]\n%i\n", user_controls[ACTION_MODIFIER2].mod);
+	fprintf(fp, "[ACTION_MODIFIER3.device]\n%i\n", user_controls[ACTION_MODIFIER3].device);
+	fprintf(fp, "[ACTION_MODIFIER3.id]\n%i\n", user_controls[ACTION_MODIFIER3].id);
+	fprintf(fp, "[ACTION_MODIFIER3.mod]\n%i\n", user_controls[ACTION_MODIFIER3].mod);
+	fprintf(fp, "[ACTION_MODIFIER4.device]\n%i\n", user_controls[ACTION_MODIFIER4].device);
+	fprintf(fp, "[ACTION_MODIFIER4.id]\n%i\n", user_controls[ACTION_MODIFIER4].id);
+	fprintf(fp, "[ACTION_MODIFIER4.mod]\n%i\n", user_controls[ACTION_MODIFIER4].mod);
+	fprintf(fp, "[ACTION_SCROLL_UP.device]\n%i\n", user_controls[ACTION_SCROLL_UP].device);
+	fprintf(fp, "[ACTION_SCROLL_UP.id]\n%i\n", user_controls[ACTION_SCROLL_UP].id);
+	fprintf(fp, "[ACTION_SCROLL_UP.mod]\n%i\n", user_controls[ACTION_SCROLL_UP].mod);
+	fprintf(fp, "[ACTION_SCROLL_DOWN.device]\n%i\n", user_controls[ACTION_SCROLL_DOWN].device);
+	fprintf(fp, "[ACTION_SCROLL_DOWN.id]\n%i\n", user_controls[ACTION_SCROLL_DOWN].id);
+	fprintf(fp, "[ACTION_SCROLL_DOWN.mod]\n%i\n", user_controls[ACTION_SCROLL_DOWN].mod);
+	fprintf(fp, "[ACTION_PRIMARY_CLICK.device]\n%i\n", user_controls[ACTION_PRIMARY_CLICK].device);
+	fprintf(fp, "[ACTION_PRIMARY_CLICK.id]\n%i\n", user_controls[ACTION_PRIMARY_CLICK].id);
+	fprintf(fp, "[ACTION_PRIMARY_CLICK.mod]\n%i\n", user_controls[ACTION_PRIMARY_CLICK].mod);
 
 	fclose(fp);
 
