@@ -24,14 +24,26 @@
  */
 
 /* Defines */
-#define RESOURCE_FILE ".gnurobborc"
+#ifdef PLATFORM_GP2X
+	#define RESOURCE_FILE "gnurobborc"
+#else
+	#define RESOURCE_FILE ".gnurobborc"
+#endif
+#define RCFILE_SAVE_ON_EXIT 0
+#define RCFILE_SAVE_ON_CHANGE 1
 
 /* Variables */
+char path_resource_file[100];
 
+struct {
+	int save_frequency;
+} rcfile;
+
+int temp_rcfile_save_frequency;
 
 /* Function prototypes */
 int read_resource_file(char *filename);
-int save_resource_file(char *filename);
+int save_resource_file(char *filename, int inform);
 
 
 
