@@ -40,7 +40,9 @@
 #define BMP_CIPHERS32 "ciphers32.bmp"
 #define BMP_ALPHA16 "alpha16.bmp"
 #define BMP_ALPHA32 "alpha32.bmp"
-#define ROBBO_FONT "robbo.ttf"
+#define TTF_ROBBO "robbo.ttf"
+#define BMP_FONT_LOWRES "font12x16.bmp"
+#define BMP_FONT_HIRES "font24x32.bmp"
 
 /* Variables */
 char *data_state[] = {"DATA_UNREAD", "DATA_READING", "DATA_READ", "DATA_INCOMPLETE"};
@@ -48,7 +50,48 @@ char *data_state[] = {"DATA_UNREAD", "DATA_READING", "DATA_READ", "DATA_INCOMPLE
 /* Function prototypes */
 void read_skin_project_colours(void);
 
+#ifdef HILDON
+SDL_Surface * arrows[ARROW_SIZE];
 
+void loadArrows() {
+  int count;
+	char filename[256];
+  SDL_Surface * tmp;
+
+  for(count=0;count<ARROW_SIZE;count++) {
+		strcpy(filename, PACKAGE_DATA_DIR);
+    strcat(filename, "/n8x0/");
+    switch(count) {
+      case ARROW_UP:
+        strcat(filename, "arrow2_up_green.bmp");
+        break;
+      case ARROW_DOWN:
+        strcat(filename, "arrow2_down_green.bmp");
+        break;
+      case ARROW_LEFT:
+        strcat(filename, "arrow2_left_green.bmp");
+        break;
+      case ARROW_RIGHT:
+        strcat(filename, "arrow2_right_green.bmp");
+        break;
+    }
+    tmp = SDL_LoadBMP(filename);
+    if(tmp == NULL) {
+	    printf("Unable to load bitmap: %s\n", SDL_GetError());
+      exit(1);
+    }
+    /* Set the transparent colour */
+    if(SDL_SetColorKey(tmp, SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB(tmp->format, 0xff, 0xff, 0xff)) < 0) {
+      fprintf(stdout,"Cannot set surface colour key: %s\n", SDL_GetError());
+      exit(1);
+    }
+    /* Convert the loaded surface to the same pixel format as the screen for fast blitting */
+    arrows[count] = SDL_DisplayFormat(tmp);
+    SDL_FreeSurface(tmp);
+  }
+}
+
+#endif
 /***************************************************************************
  * Find All Skins                                                          *
  ***************************************************************************/
@@ -430,322 +473,6 @@ void read_skin_project_colours(void) {
 	#endif
 }
 
-/***************************************************************************
- * Read Skin Project Colours                                               *
- ***************************************************************************/
-/* Reads the projects colours from the skinrc file.
-   Anything not found will be initialised with some defaults.
-   Actually this function is a bit overkill for data that has a default but
-   nevermind! I've started so I'll finish :) */
-
-/*void read_skin_project_colours(void) {	Redundant: Rewritten above.
-	FILE *fp;
-	char filename[256], line[1024];	/$ 1024 because if fgets returns a partial line then line_count will be incremented again $/
-	int count, count2, line_count = 0;
-	int value_read[1];
-	int project_colours_read = DATA_UNREAD;
-	int background_read = DATA_UNREAD;
-	int version_text_read = DATA_UNREAD;
-	int general_text_read = DATA_UNREAD;
-	int menu_text_read = DATA_UNREAD;
-	int menu_selected_text_read = DATA_UNREAD;
-	int menu_selected_limit_text_read = DATA_UNREAD;
-	int menu_greyed_text_read = DATA_UNREAD;
-	int default_controls_text_read = DATA_UNREAD;
-	int credits_text_read = DATA_UNREAD;
-	int help_tile_read = DATA_UNREAD;
-	int fade_read = DATA_UNREAD;
-	int author_text_read = DATA_UNREAD;
-
-	#ifdef DEBUG_READ_SKINRC_FILE
-		printf("*** Start %s ***\n", __func__);
-	#endif
-
-	for (count = 0; count < skin_count; count++) {
-		/$ Reset these because we'll be using them more than once $/
-		line_count = 0;
-		project_colours_read = DATA_UNREAD;
-		background_read = DATA_UNREAD;
-		version_text_read = DATA_UNREAD;
-		general_text_read = DATA_UNREAD;
-		menu_text_read = DATA_UNREAD;
-		menu_selected_text_read = DATA_UNREAD;
-		menu_selected_limit_text_read = DATA_UNREAD;
-		menu_greyed_text_read = DATA_UNREAD;
-		default_controls_text_read = DATA_UNREAD;
-		credits_text_read = DATA_UNREAD;
-		help_tile_read = DATA_UNREAD;
-		fade_read = DATA_UNREAD;
-		author_text_read = DATA_UNREAD;
-		/$ Initialise everything to some defaults $/
-		skins[count].background_colour = BACKGROUND_COLOUR;
-		skins[count].version_text_colour = VERSION_TEXT_COLOUR;
-		skins[count].general_text_colour = GENERAL_TEXT_COLOUR;
-		skins[count].menu_text_colour = MENU_TEXT_COLOUR;
-		skins[count].menu_selected_text_colour = MENU_SELECTED_TEXT_COLOUR;
-		skins[count].menu_selected_limit_text_colour = MENU_SELECTED_LIMIT_TEXT_COLOUR;
-		skins[count].menu_greyed_text_colour = MENU_GREYED_TEXT_COLOUR;
-		skins[count].default_controls_text_colour = DEFAULT_CONTROLS_TEXT_COLOUR;
-		skins[count].credits_text_colour = CREDITS_TEXT_COLOUR;
-		skins[count].help_tile_colour = HELP_TILE_COLOUR;
-		skins[count].fade_colour = FADE_COLOUR;
-		skins[count].author_text_colour = AUTHOR_TEXT_COLOUR;
-		/$ Attempt to open the skinrc file $/
-		strcpy(filename, PACKAGE_DATA_DIR SKINS_DIR);
-		strcat(filename, skins[count].foldername);
-		strcat(filename, "/");
-		strcat(filename, SKINRC_FILE);
-		if ((fp = fopen(filename, "r")) == NULL) {
-			fprintf(stdout, "Cannot read from file %s\n", filename);
-		} else {
-			/$ Read lines including the [CR]LFs until we have what we need or reach the end of file (line == NULL).
-			   We are looking for all of these things :-
-			   [project_colours]
-			   [background]
-			   [version_text]
-			   [general_text]
-			   [menu_text]
-			   [menu_selected_text]
-			   [menu_selected_limit_text]
-			   [menu_greyed_text]
-			   [default_controls_text]
-			   [credits_text]
-			   [help_tile]
-			   [fade]
-			   [author_text]
-			   [project_colours_end] $/
-			while ((fgets(line, 1024, fp)) != NULL) {
-				line_count++;
-				/$ fgets will get [CR]LFs too so they should be removed $/
-				for (count2 = strlen(line) - 1; count2 >= 0; count2--) if (line[count2] == 13 || line[count2] == 10) line[count2] = 0;
-				/$ Use the tags as switches so we know what to read $/
-				/$ NOTE: Reading consecutive (dataless) tags means that the previous tag may still be marked as DATA_READING $/
-				if ((!strncmp(line, "[project_colours]", 17)) || 
-				(!strncmp(line, "[background]", 12) && project_colours_read != DATA_UNREAD) || 
-				(!strncmp(line, "[version_text]", 14) && project_colours_read != DATA_UNREAD) || 
-				(!strncmp(line, "[general_text]", 14) && project_colours_read != DATA_UNREAD) || 
-				(!strncmp(line, "[menu_text]", 11) && project_colours_read != DATA_UNREAD) || 
-				(!strncmp(line, "[menu_selected_text]", 20) && project_colours_read != DATA_UNREAD) || 
-				(!strncmp(line, "[menu_selected_limit_text]", 26) && project_colours_read != DATA_UNREAD) || 
-				(!strncmp(line, "[menu_greyed_text]", 18) && project_colours_read != DATA_UNREAD) || 
-				(!strncmp(line, "[default_controls_text]", 23) && project_colours_read != DATA_UNREAD) || 
-				(!strncmp(line, "[credits_text]", 14) && project_colours_read != DATA_UNREAD) || 
-				(!strncmp(line, "[help_tile]", 11) && project_colours_read != DATA_UNREAD) || 
-				(!strncmp(line, "[fade]", 6) && project_colours_read != DATA_UNREAD) || 
-				(!strncmp(line, "[author_text]", 13) && project_colours_read != DATA_UNREAD) || 
-				(!strncmp(line, "[project_colours_end]", 21))) {
-					/$ If a tag is being read and we read another tag then mark it as something else $/
-					if (project_colours_read == DATA_READING) {
-						project_colours_read = DATA_READ;	/$ A dataless tag $/
-					} else if (background_read == DATA_READING) {
-						background_read = DATA_INCOMPLETE;
-					} else if (version_text_read == DATA_READING) {
-						version_text_read = DATA_INCOMPLETE;
-					} else if (general_text_read == DATA_READING) {
-						general_text_read = DATA_INCOMPLETE;
-					} else if (menu_text_read == DATA_READING) {
-						menu_text_read = DATA_INCOMPLETE;
-					} else if (menu_selected_text_read == DATA_READING) {
-						menu_selected_text_read = DATA_INCOMPLETE;
-					} else if (menu_selected_limit_text_read == DATA_READING) {
-						menu_selected_limit_text_read = DATA_INCOMPLETE;
-					} else if (menu_greyed_text_read == DATA_READING) {
-						menu_greyed_text_read = DATA_INCOMPLETE;
-					} else if (default_controls_text_read == DATA_READING) {
-						default_controls_text_read = DATA_INCOMPLETE;
-					} else if (credits_text_read == DATA_READING) {
-						credits_text_read = DATA_INCOMPLETE;
-					} else if (help_tile_read == DATA_READING) {
-						help_tile_read = DATA_INCOMPLETE;
-					} else if (fade_read == DATA_READING) {
-						fade_read = DATA_INCOMPLETE;
-					} else if (author_text_read == DATA_READING) {
-						author_text_read = DATA_INCOMPLETE;
-					}
-					/$ If the found tag is DATA_UNREAD then mark it for DATA_READING $/
-					if (!strncmp(line, "[project_colours]", 17)) {
-						if (project_colours_read == DATA_UNREAD) project_colours_read = DATA_READING;
-					} else if (!strncmp(line, "[background]", 12)) {
-						if (background_read == DATA_UNREAD) background_read = DATA_READING;
-					} else if (!strncmp(line, "[version_text]", 14)) {
-						if (version_text_read == DATA_UNREAD) version_text_read = DATA_READING;
-					} else if (!strncmp(line, "[general_text]", 14)) {
-						if (general_text_read == DATA_UNREAD) general_text_read = DATA_READING;
-					} else if (!strncmp(line, "[menu_text]", 11)) {
-						if (menu_text_read == DATA_UNREAD) menu_text_read = DATA_READING;
-					} else if (!strncmp(line, "[menu_selected_text]", 20)) {
-						if (menu_selected_text_read == DATA_UNREAD) menu_selected_text_read = DATA_READING;
-					} else if (!strncmp(line, "[menu_selected_limit_text]", 26)) {
-						if (menu_selected_limit_text_read == DATA_UNREAD) menu_selected_limit_text_read = DATA_READING;
-					} else if (!strncmp(line, "[menu_greyed_text]", 18)) {
-						if (menu_greyed_text_read == DATA_UNREAD) menu_greyed_text_read = DATA_READING;
-					} else if (!strncmp(line, "[default_controls_text]", 23)) {
-						if (default_controls_text_read == DATA_UNREAD) default_controls_text_read = DATA_READING;
-					} else if (!strncmp(line, "[credits_text]", 14)) {
-						if (credits_text_read == DATA_UNREAD) credits_text_read = DATA_READING;
-					} else if (!strncmp(line, "[help_tile]", 11)) {
-						if (help_tile_read == DATA_UNREAD) help_tile_read = DATA_READING;
-					} else if (!strncmp(line, "[fade]", 6)) {
-						if (fade_read == DATA_UNREAD) fade_read = DATA_READING;
-					} else if (!strncmp(line, "[author_text]", 13)) {
-						if (author_text_read == DATA_UNREAD) author_text_read = DATA_READING;
-					} else if (!strncmp(line, "[project_colours_end]", 21)) {
-						break;	/$ Stop reading the file now $/
-					}
-				} else {
-					/$ It's not a tag but data (or newline) so we need to find what to do with it $/
-					if (background_read == DATA_READING) {
-						if (sscanf(line, "%x", &value_read[0]) == 1) {
-							skins[count].background_colour = value_read[0];
-							background_read = DATA_READ;
-							#ifdef DEBUG_READ_SKINRC_FILE
-								printf("%i: background_colour=0x%06X\n", line_count, value_read[0]);
-							#endif
-						} else {
-							background_read = DATA_INCOMPLETE;	/$ Read once, found to be incomplete, the end :) $/
-						}
-					} else if (version_text_read == DATA_READING) {
-						if (sscanf(line, "%x", &value_read[0]) == 1) {
-							skins[count].version_text_colour = value_read[0];
-							version_text_read = DATA_READ;
-							#ifdef DEBUG_READ_SKINRC_FILE
-								printf("%i: version_text_colour=0x%06X\n", line_count, value_read[0]);
-							#endif
-						} else {
-							version_text_read = DATA_INCOMPLETE;
-						}
-					} else if (general_text_read == DATA_READING) {
-						if (sscanf(line, "%x", &value_read[0]) == 1) {
-							skins[count].general_text_colour = value_read[0];
-							general_text_read = DATA_READ;
-							#ifdef DEBUG_READ_SKINRC_FILE
-								printf("%i: general_text_colour=0x%06X\n", line_count, value_read[0]);
-							#endif
-						} else {
-							general_text_read = DATA_INCOMPLETE;
-						}
-					} else if (menu_text_read == DATA_READING) {
-						if (sscanf(line, "%x", &value_read[0]) == 1) {
-							skins[count].menu_text_colour = value_read[0];
-							menu_text_read = DATA_READ;
-							#ifdef DEBUG_READ_SKINRC_FILE
-								printf("%i: menu_text_colour=0x%06X\n", line_count, value_read[0]);
-							#endif
-						} else {
-							menu_text_read = DATA_INCOMPLETE;
-						}
-					} else if (menu_selected_text_read == DATA_READING) {
-						if (sscanf(line, "%x", &value_read[0]) == 1) {
-							skins[count].menu_selected_text_colour = value_read[0];
-							menu_selected_text_read = DATA_READ;
-							#ifdef DEBUG_READ_SKINRC_FILE
-								printf("%i: menu_selected_text_colour=0x%06X\n", line_count, value_read[0]);
-							#endif
-						} else {
-							menu_selected_text_read = DATA_INCOMPLETE;
-						}
-					} else if (menu_selected_limit_text_read == DATA_READING) {
-						if (sscanf(line, "%x", &value_read[0]) == 1) {
-							skins[count].menu_selected_limit_text_colour = value_read[0];
-							menu_selected_limit_text_read = DATA_READ;
-							#ifdef DEBUG_READ_SKINRC_FILE
-								printf("%i: menu_selected_limit_text_colour=0x%06X\n", line_count, value_read[0]);
-							#endif
-						} else {
-							menu_selected_limit_text_read = DATA_INCOMPLETE;
-						}
-					} else if (menu_greyed_text_read == DATA_READING) {
-						if (sscanf(line, "%x", &value_read[0]) == 1) {
-							skins[count].menu_greyed_text_colour = value_read[0];
-							menu_greyed_text_read = DATA_READ;
-							#ifdef DEBUG_READ_SKINRC_FILE
-								printf("%i: menu_greyed_text_colour=0x%06X\n", line_count, value_read[0]);
-							#endif
-						} else {
-							menu_greyed_text_read = DATA_INCOMPLETE;
-						}
-					} else if (default_controls_text_read == DATA_READING) {
-						if (sscanf(line, "%x", &value_read[0]) == 1) {
-							skins[count].default_controls_text_colour = value_read[0];
-							default_controls_text_read = DATA_READ;
-							#ifdef DEBUG_READ_SKINRC_FILE
-								printf("%i: default_controls_text_colour=0x%06X\n", line_count, value_read[0]);
-							#endif
-						} else {
-							default_controls_text_read = DATA_INCOMPLETE;
-						}
-					} else if (credits_text_read == DATA_READING) {
-						if (sscanf(line, "%x", &value_read[0]) == 1) {
-							skins[count].credits_text_colour = value_read[0];
-							credits_text_read = DATA_READ;
-							#ifdef DEBUG_READ_SKINRC_FILE
-								printf("%i: credits_text_colour=0x%06X\n", line_count, value_read[0]);
-							#endif
-						} else {
-							credits_text_read = DATA_INCOMPLETE;
-						}
-					} else if (help_tile_read == DATA_READING) {
-						if (sscanf(line, "%x", &value_read[0]) == 1) {
-							skins[count].help_tile_colour = value_read[0];
-							help_tile_read = DATA_READ;
-							#ifdef DEBUG_READ_SKINRC_FILE
-								printf("%i: help_tile_colour=0x%06X\n", line_count, value_read[0]);
-							#endif
-						} else {
-							help_tile_read = DATA_INCOMPLETE;
-						}
-					} else if (fade_read == DATA_READING) {
-						if (sscanf(line, "%x", &value_read[0]) == 1) {
-							skins[count].fade_colour = value_read[0];
-							fade_read = DATA_READ;
-							#ifdef DEBUG_READ_SKINRC_FILE
-								printf("%i: fade_colour=0x%06X\n", line_count, value_read[0]);
-							#endif
-						} else {
-							fade_read = DATA_INCOMPLETE;
-						}
-					} else if (author_text_read == DATA_READING) {
-						if (sscanf(line, "%x", &value_read[0]) == 1) {
-							skins[count].author_text_colour = value_read[0];
-							author_text_read = DATA_READ;
-							#ifdef DEBUG_READ_SKINRC_FILE
-								printf("%i: author_text_colour=0x%06X\n", line_count, value_read[0]);
-							#endif
-						} else {
-							author_text_read = DATA_INCOMPLETE;
-						}
-					}
-				}
-			}
-
-			/$ Dump info to the console $/
-			fprintf(stdout, "Skin found: %s\n", skins[count].name);
-
-			fclose(fp);
-
-			#ifdef DEBUG_READ_SKINRC_FILE
-				printf("project_colours_read=%s\n", data_state[project_colours_read]);
-				printf("background_read=%s\n", data_state[background_read]);
-				printf("version_text_read=%s\n", data_state[version_text_read]);
-				printf("general_text_read=%s\n", data_state[general_text_read]);
-				printf("menu_text_read=%s\n", data_state[menu_text_read]);
-				printf("menu_selected_text_read=%s\n", data_state[menu_selected_text_read]);
-				printf("menu_selected_limit_text_read=%s\n", data_state[menu_selected_limit_text_read]);
-				printf("menu_greyed_text_read=%s\n", data_state[menu_greyed_text_read]);
-				printf("default_controls_text_read=%s\n", data_state[default_controls_text_read]);
-				printf("credits_text_read=%s\n", data_state[credits_text_read]);
-				printf("help_tile_read=%s\n", data_state[help_tile_read]);
-				printf("fade_read=%s\n", data_state[fade_read]);
-				printf("author_text_read=%s\n\n", data_state[author_text_read]);
-			#endif
-
-		}
-	}
-	#ifdef DEBUG_READ_SKINRC_FILE
-		printf("*** Stop %s ***\n", __func__);
-	#endif
-} */
-
 /***************************************************************************
  * Sort Skins                                                              *
  ***************************************************************************/
@@ -946,11 +673,11 @@ void load_selected_skin(void) {
 	Uint32 rmask, gmask, bmask, amask;
 	
 	/* Free SDL surfaces first if they are not NULL */
-	if (icons != NULL) SDL_FreeSurface(icons);
-	if (ciphers != NULL) SDL_FreeSurface(ciphers);
-	if (alpha != NULL) SDL_FreeSurface(alpha);
-	if (font != NULL) TTF_CloseFont(font);
-	if (icon != NULL) SDL_FreeSurface(icon);
+	if (icons) SDL_FreeSurface(icons);
+	if (ciphers) SDL_FreeSurface(ciphers);
+	if (alpha) SDL_FreeSurface(alpha);
+	if (icon) SDL_FreeSurface(icon);
+	if (font) TTF_CloseFont(font);
 
 	for (count = 0; count < 4; count++) {
 		strcpy(filename, PACKAGE_DATA_DIR SKINS_DIR);
@@ -979,14 +706,23 @@ void load_selected_skin(void) {
 			}
 		} else if (count == 3) {
 			/* Load the font */ 	
-			strcat(filename, ROBBO_FONT);
-			if (video.field_size == 16) {
-				fontsize = FONTSIZE16;
-			} else if (video.field_size == 32) {
-				fontsize = FONTSIZE32;
-			}
+			#ifdef USE_PIXMAP_FONT
+				fontsize = 0;	/* Not used */
+				if (video.field_size == 16) {
+					strcat(filename, BMP_FONT_LOWRES);
+				} else if (video.field_size == 32) {
+					strcat(filename, BMP_FONT_HIRES);
+				}
+			#else
+				strcat(filename, TTF_ROBBO);
+				if (video.field_size == 16) {
+					fontsize = FONTSIZE16;
+				} else if (video.field_size == 32) {
+					fontsize = FONTSIZE32;
+				}
+			#endif
 			if ((font = TTF_OpenFont(filename, fontsize)) == NULL) {
-				fprintf(stdout, "Cannot load robbo font: %s\n", filename);
+				fprintf(stdout, "Cannot load font: %s\n", filename);
 				exit(1);
 			}
 		}
@@ -1003,11 +739,11 @@ void load_selected_skin(void) {
 			}
 			/* Convert the loaded surface to the same pixel format as the screen for fast blitting */
 			converted = SDL_DisplayFormat(unconverted);
+			SDL_FreeSurface(unconverted);
 			if (converted == NULL) {
 				fprintf(stdout, "Cannot create surface: %s\n", SDL_GetError());
 				exit(1);
 			}
-			SDL_FreeSurface(unconverted);
 			if (count == 0) {
 				icons = converted;
 			} else if (count == 1) {
@@ -1062,6 +798,10 @@ void load_selected_skin(void) {
 	}
 	SDL_BlitSurface(icons, &srcrect, icon, NULL);
 
+#ifdef HILDON	
+  loadArrows();
+#endif
+
 }
 
 /***************************************************************************
